@@ -25,8 +25,8 @@ exports.start = function(argv, stdin, stdout) {
   stdin = stdin || process.stdin;
   stdout = stdout || process.stdout;
 
-  var args = ['--debug-brk'].concat(argv),
-      interface_ = new Interface(stdin, stdout, args);
+  const args = [`--debug-brk=${exports.port}`].concat(argv);
+  const interface_ = new Interface(stdin, stdout, args);
 
   stdin.resume();
 
@@ -40,7 +40,7 @@ exports.start = function(argv, stdin, stdout) {
   });
 };
 
-exports.port = 5858;
+exports.port = process.debugPort;
 
 
 //
@@ -142,7 +142,7 @@ Protocol.prototype.serialize = function(req) {
 const NO_FRAME = -1;
 
 function Client() {
-  net.Stream.call(this);
+  net.Socket.call(this);
   var protocol = this.protocol = new Protocol(this);
   this._reqCallbacks = [];
   var socket = this;
@@ -159,9 +159,9 @@ function Client() {
     protocol.execute(d);
   });
 
-  protocol.onResponse = this._onResponse.bind(this);
+  protocol.onResponse = (res) => this._onResponse(res);
 }
-inherits(Client, net.Stream);
+inherits(Client, net.Socket);
 exports.Client = Client;
 
 
@@ -186,7 +186,7 @@ Client.prototype._addScript = function(desc) {
   this.scripts[desc.id] = desc;
   if (desc.name) {
     desc.isNative = (desc.name.replace('.js', '') in natives) ||
-                    desc.name == 'node.js';
+                    desc.name === 'node.js';
   }
 };
 
@@ -197,11 +197,11 @@ Client.prototype._removeScript = function(desc) {
 
 
 Client.prototype._onResponse = function(res) {
-  var cb,
-      index = -1;
+  var cb;
+  var index = -1;
 
   this._reqCallbacks.some(function(fn, i) {
-    if (fn.request_seq == res.body.request_seq) {
+    if (fn.request_seq === res.body.request_seq) {
       cb = fn;
       index = i;
       return true;
@@ -211,25 +211,25 @@ Client.prototype._onResponse = function(res) {
   var self = this;
   var handled = false;
 
-  if (res.headers.Type == 'connect') {
+  if (res.headers.Type === 'connect') {
     // Request a list of scripts for our own storage.
     self.reqScripts();
     self.emit('ready');
     handled = true;
 
-  } else if (res.body && res.body.event == 'break') {
+  } else if (res.body && res.body.event === 'break') {
     this.emit('break', res.body);
     handled = true;
 
-  } else if (res.body && res.body.event == 'exception') {
+  } else if (res.body && res.body.event === 'exception') {
     this.emit('exception', res.body);
     handled = true;
 
-  } else if (res.body && res.body.event == 'afterCompile') {
+  } else if (res.body && res.body.event === 'afterCompile') {
     this._addHandle(res.body.body.script);
     handled = true;
 
-  } else if (res.body && res.body.event == 'scriptCollected') {
+  } else if (res.body && res.body.event === 'scriptCollected') {
     // ???
     this._removeScript(res.body.body.script);
     handled = true;
@@ -295,11 +295,11 @@ Client.prototype.reqLookup = function(refs, cb) {
 };
 
 Client.prototype.reqScopes = function(cb) {
-  var self = this,
-      req = {
-        command: 'scopes',
-        arguments: {}
-      };
+  const self = this;
+  const req = {
+    command: 'scopes',
+    arguments: {}
+  };
 
   cb = cb || function() {};
   this.req(req, function(err, res) {
@@ -327,7 +327,7 @@ Client.prototype.reqScopes = function(cb) {
 Client.prototype.reqEval = function(expression, cb) {
   var self = this;
 
-  if (this.currentFrame == NO_FRAME) {
+  if (this.currentFrame === NO_FRAME) {
     // Only need to eval in global scope.
     this.reqFrameEval(expression, NO_FRAME, cb);
     return;
@@ -357,7 +357,7 @@ Client.prototype.reqEval = function(expression, cb) {
 
 // Finds the first scope in the array in which the expression evals.
 Client.prototype._reqFramesEval = function(expression, evalFrames, cb) {
-  if (evalFrames.length == 0) {
+  if (evalFrames.length === 0) {
     // Just eval in global scope.
     this.reqFrameEval(expression, NO_FRAME, cb);
     return;
@@ -381,7 +381,7 @@ Client.prototype.reqFrameEval = function(expression, frame, cb) {
     arguments: { expression: expression }
   };
 
-  if (frame == NO_FRAME) {
+  if (frame === NO_FRAME) {
     req.arguments.global = true;
   } else {
     req.arguments.frame = frame;
@@ -462,7 +462,7 @@ Client.prototype.setBreakpoint = function(req, cb) {
 };
 
 Client.prototype.clearBreakpoint = function(req, cb) {
-  var req = {
+  req = {
     command: 'clearbreakpoint',
     arguments: req
   };
@@ -525,12 +525,12 @@ Client.prototype.mirrorObject = function(handle, depth, cb) {
         return;
       }
 
-      var mirror,
-          waiting = 1;
+      var mirror;
+      var waiting = 1;
 
-      if (handle.className == 'Array') {
+      if (handle.className === 'Array') {
         mirror = [];
-      } else if (handle.className == 'Date') {
+      } else if (handle.className === 'Date') {
         mirror = new Date(handle.value);
       } else {
         mirror = {};
@@ -547,8 +547,8 @@ Client.prototype.mirrorObject = function(handle, depth, cb) {
           mirrorValue = '[?]';
         }
 
-        if (Array.isArray(mirror) && typeof prop.name !== 'number') {
-          // Skip the 'length' property.
+        // Skip the 'length' property.
+        if (Array.isArray(mirror) && prop.name === 'length') {
           return;
         }
 
@@ -567,13 +567,13 @@ Client.prototype.mirrorObject = function(handle, depth, cb) {
 
       waitForOthers();
       function waitForOthers() {
-        if (--waiting === 0 && cb) {
+        if (--waiting === 0) {
           keyValues.forEach(function(kv) {
             mirror[kv.name] = kv.value;
           });
           cb(null, mirror);
         }
-      };
+      }
     });
     return;
   } else if (handle.type === 'function') {
@@ -657,6 +657,7 @@ const commands = [
     'unwatch',
     'watchers',
     'repl',
+    'exec',
     'restart',
     'kill',
     'list',
@@ -676,8 +677,8 @@ var helpMessage = 'Commands: ' + commands.map(function(group) {
 function SourceUnderline(sourceText, position, repl) {
   if (!sourceText) return '';
 
-  var head = sourceText.slice(0, position),
-      tail = sourceText.slice(position);
+  const head = sourceText.slice(0, position);
+  var tail = sourceText.slice(position);
 
   // Colourize char if stdout supports colours
   if (repl.useColors) {
@@ -697,8 +698,8 @@ function SourceInfo(body) {
 
   if (body.script) {
     if (body.script.name) {
-      var name = body.script.name,
-          dir = path.resolve() + '/';
+      var name = body.script.name;
+      const dir = path.resolve() + '/';
 
       // Change path to relative, if possible
       if (name.indexOf(dir) === 0) {
@@ -733,7 +734,7 @@ function Interface(stdin, stdout, args) {
     prompt: 'debug> ',
     input: this.stdin,
     output: this.stdout,
-    eval: this.controlEval.bind(this),
+    eval: (code, ctx, file, cb) => this.controlEval(code, ctx, file, cb),
     useGlobal: false,
     ignoreUndefined: true
   };
@@ -764,15 +765,25 @@ function Interface(stdin, stdout, args) {
   });
 
   // Handle all possible exits
-  process.on('exit', this.killChild.bind(this));
+  process.on('exit', () => this.killChild());
   process.once('SIGTERM', process.exit.bind(process, 0));
   process.once('SIGHUP', process.exit.bind(process, 0));
 
   var proto = Interface.prototype;
-  const ignored = ['pause', 'resume', 'exitRepl', 'handleBreak',
-                   'requireConnection', 'killChild', 'trySpawn',
-                   'controlEval', 'debugEval', 'print', 'childPrint',
-                   'clearline'];
+  const ignored = [
+    'pause',
+    'resume',
+    'exitRepl',
+    'handleBreak',
+    'requireConnection',
+    'killChild',
+    'trySpawn',
+    'controlEval',
+    'debugEval',
+    'print',
+    'childPrint',
+    'clearline'
+  ];
   const shortcut = {
     'run': 'r',
     'cont': 'c',
@@ -798,7 +809,7 @@ function Interface(stdin, stdout, args) {
     } else {
       self.repl.context[key] = fn;
     }
-  };
+  }
 
   // Copy all prototype methods in repl context
   // Setup them as getters if possible
@@ -824,13 +835,7 @@ function Interface(stdin, stdout, args) {
   // Run script automatically
   this.pause();
 
-  // XXX Need to figure out why we need this delay
-  setTimeout(function() {
-
-    self.run(function() {
-      self.resume();
-    });
-  }, 10);
+  setImmediate(() => { this.run(); });
 }
 
 
@@ -949,6 +954,12 @@ Interface.prototype.controlEval = function(code, context, filename, callback) {
       }
     }
 
+    // exec process.title => exec("process.title");
+    var match = code.match(/^\s*exec\s+([^\n]*)/);
+    if (match) {
+      code = 'exec(' + JSON.stringify(match[1]) + ')';
+    }
+
     var result = vm.runInContext(code, context, filename);
 
     // Repl should not ask for next command
@@ -969,8 +980,8 @@ Interface.prototype.controlEval = function(code, context, filename, callback) {
 Interface.prototype.debugEval = function(code, context, filename, callback) {
   if (!this.requireConnection()) return;
 
-  var self = this,
-      client = this.client;
+  const self = this;
+  const client = this.client;
 
   // Repl asked for scope variables
   if (code === '.scope') {
@@ -1004,15 +1015,11 @@ Interface.prototype.debugEval = function(code, context, filename, callback) {
 // Adds spaces and prefix to number
 // maxN is a maximum number we should have space for
 function leftPad(n, prefix, maxN) {
-  var s = n.toString(),
-      nchars = Math.max(2, String(maxN).length) + 1,
-      nspaces = nchars - s.length - 1;
+  const s = n.toString();
+  const nchars = Math.max(2, String(maxN).length) + 1;
+  const nspaces = nchars - s.length - 1;
 
-  for (var i = 0; i < nspaces; i++) {
-    prefix += ' ';
-  }
-
-  return prefix + s;
+  return prefix + ' '.repeat(nspaces) + s;
 }
 
 
@@ -1078,10 +1085,10 @@ Interface.prototype.list = function(delta) {
 
   delta || (delta = 5);
 
-  var self = this,
-      client = this.client,
-      from = client.currentSourceLine - delta + 1,
-      to = client.currentSourceLine + delta + 1;
+  const self = this;
+  const client = this.client;
+  const from = client.currentSourceLine - delta + 1;
+  const to = client.currentSourceLine + delta + 1;
 
   self.pause();
   client.reqSource(from, to, function(err, res) {
@@ -1096,14 +1103,14 @@ Interface.prototype.list = function(delta) {
       var lineno = res.fromLine + i + 1;
       if (lineno < from || lineno > to) continue;
 
-      var current = lineno == 1 + client.currentSourceLine,
-          breakpoint = client.breakpoints.some(function(bp) {
-            return (bp.scriptReq === client.currentScript ||
-                    bp.script === client.currentScript) &&
-                    bp.line == lineno;
-          });
+      const current = lineno === 1 + client.currentSourceLine;
+      const breakpoint = client.breakpoints.some(function(bp) {
+        return (bp.scriptReq === client.currentScript ||
+                bp.script === client.currentScript) &&
+                bp.line === lineno;
+      });
 
-      if (lineno == 1) {
+      if (lineno === 1) {
         // The first line needs to have the module wrapper filtered out of
         // it.
         var wrapper = Module.wrapper[0];
@@ -1139,8 +1146,8 @@ Interface.prototype.list = function(delta) {
 Interface.prototype.backtrace = function() {
   if (!this.requireConnection()) return;
 
-  var self = this,
-      client = this.client;
+  const self = this;
+  const client = this.client;
 
   self.pause();
   client.fullTrace(function(err, bt) {
@@ -1150,11 +1157,11 @@ Interface.prototype.backtrace = function() {
       return;
     }
 
-    if (bt.totalFrames == 0) {
+    if (bt.totalFrames === 0) {
       self.print('(empty stack)');
     } else {
-      var trace = [],
-          firstFrameNative = bt.frames[0].script.isNative;
+      const trace = [];
+      const firstFrameNative = bt.frames[0].script.isNative;
 
       for (var i = 0; i < bt.frames.length; i++) {
         var frame = bt.frames[i];
@@ -1183,19 +1190,19 @@ Interface.prototype.backtrace = function() {
 Interface.prototype.scripts = function() {
   if (!this.requireConnection()) return;
 
-  var client = this.client,
-      displayNatives = arguments[0] || false,
-      scripts = [];
+  const client = this.client;
+  const displayNatives = arguments[0] || false;
+  const scripts = [];
 
   this.pause();
   for (var id in client.scripts) {
     var script = client.scripts[id];
     if (script !== null && typeof script === 'object' && script.name) {
       if (displayNatives ||
-          script.name == client.currentScript ||
+          script.name === client.currentScript ||
           !script.isNative) {
         scripts.push(
-            (script.name == client.currentScript ? '* ' : '  ') +
+            (script.name === client.currentScript ? '* ' : '  ') +
             id + ': ' +
             path.basename(script.name)
         );
@@ -1323,9 +1330,9 @@ Interface.prototype.setBreakpoint = function(script, line,
                                              condition, silent) {
   if (!this.requireConnection()) return;
 
-  var self = this,
-      scriptId,
-      ambiguous;
+  const self = this;
+  var scriptId;
+  var ambiguous;
 
   // setBreakpoint() should insert breakpoint on current line
   if (script === undefined) {
@@ -1345,9 +1352,10 @@ Interface.prototype.setBreakpoint = function(script, line,
     return;
   }
 
-  if (/\(\)$/.test(script)) {
+  let req;
+  if (script.endsWith('()')) {
     // setBreakpoint('functionname()');
-    var req = {
+    req = {
       type: 'function',
       target: script.replace(/\(\)$/, ''),
       condition: condition
@@ -1373,7 +1381,6 @@ Interface.prototype.setBreakpoint = function(script, line,
     if (ambiguous) return this.error('Script name is ambiguous');
     if (line <= 0) return this.error('Line should be a positive value');
 
-    var req;
     if (scriptId) {
       req = {
         type: 'scriptId',
@@ -1429,10 +1436,10 @@ Interface.prototype.setBreakpoint = function(script, line,
 Interface.prototype.clearBreakpoint = function(script, line) {
   if (!this.requireConnection()) return;
 
-  var ambiguous,
-      breakpoint,
-      scriptId,
-      index;
+  var ambiguous;
+  var breakpoint;
+  var scriptId;
+  var index;
 
   this.client.breakpoints.some(function(bp, i) {
     if (bp.scriptId === script ||
@@ -1474,10 +1481,8 @@ Interface.prototype.clearBreakpoint = function(script, line) {
     return this.error('Breakpoint not found on line ' + line);
   }
 
-  var self = this,
-      req = {
-        breakpoint: breakpoint
-      };
+  var self = this;
+  const req = {breakpoint};
 
   self.pause();
   self.client.clearBreakpoint(req, function(err, res) {
@@ -1513,8 +1518,8 @@ Interface.prototype.breakpoints = function() {
 Interface.prototype.pause_ = function() {
   if (!this.requireConnection()) return;
 
-  var self = this,
-      cmd = 'process._debugPause();';
+  const self = this;
+  const cmd = 'process._debugPause();';
 
   this.pause();
   this.client.reqFrameEval(cmd, NO_FRAME, function(err, res) {
@@ -1527,6 +1532,18 @@ Interface.prototype.pause_ = function() {
 };
 
 
+// execute expression
+Interface.prototype.exec = function(code) {
+  this.debugEval(code, null, null, (err, result) => {
+    if (err) {
+      this.error(err);
+    } else {
+      this.print(util.inspect(result, {colors: true}));
+    }
+  });
+};
+
+
 // Kill child process
 Interface.prototype.kill = function() {
   if (!this.child) return;
@@ -1568,7 +1585,8 @@ Interface.prototype.repl = function() {
   this.repl.on('exit', exitDebugRepl);
 
   // Set new
-  this.repl.eval = this.debugEval.bind(this);
+  this.repl.eval = (code, ctx, file, cb) =>
+    this.debugEval(code, ctx, file, cb);
   this.repl.context = {};
 
   // Swap history
@@ -1583,7 +1601,8 @@ Interface.prototype.repl = function() {
 // Exit debug repl
 Interface.prototype.exitRepl = function() {
   // Restore eval
-  this.repl.eval = this.controlEval.bind(this);
+  this.repl.eval = (code, ctx, file, cb) =>
+    this.controlEval(code, ctx, file, cb);
 
   // Swap history
   this.history.debug = this.repl.rli.history;
@@ -1621,18 +1640,18 @@ Interface.prototype.killChild = function() {
 
 // Spawns child process (and restores breakpoints)
 Interface.prototype.trySpawn = function(cb) {
-  var self = this,
-      breakpoints = this.breakpoints || [],
-      port = exports.port,
-      host = '127.0.0.1',
-      childArgs = this.args;
+  const self = this;
+  const breakpoints = this.breakpoints || [];
+  var port = exports.port;
+  var host = '127.0.0.1';
+  var childArgs = this.args;
 
   this.killChild();
   assert(!this.child);
 
   var isRemote = false;
   if (this.args.length === 2) {
-    var match = this.args[1].match(/^([^:]+):(\d+)$/);
+    const match = this.args[1].match(/^([^:]+):(\d+)$/);
 
     if (match) {
       // Connecting to remote debugger
@@ -1656,7 +1675,7 @@ Interface.prototype.trySpawn = function(cb) {
       }
       isRemote = true;
     } else {
-      var match = this.args[1].match(/^--port=(\d+)$/);
+      const match = this.args[1].match(/^--port=(\d+)$/);
       if (match) {
         // Start debugger on custom port
         // `node debug --port=5858 app.js`
@@ -1670,14 +1689,14 @@ Interface.prototype.trySpawn = function(cb) {
     // pipe stream into debugger
     this.child = spawn(process.execPath, childArgs);
 
-    this.child.stdout.on('data', this.childPrint.bind(this));
-    this.child.stderr.on('data', this.childPrint.bind(this));
+    this.child.stdout.on('data', (text) => this.childPrint(text));
+    this.child.stderr.on('data', (text) => this.childPrint(text));
   }
 
   this.pause();
 
-  var client = self.client = new Client(),
-      connectionAttempts = 0;
+  const client = self.client = new Client();
+  var connectionAttempts = 0;
 
   client.once('ready', function() {
     self.stdout.write(' ok\n');
@@ -1730,11 +1749,12 @@ Interface.prototype.trySpawn = function(cb) {
     client.connect(port, host);
   }
 
-  self.print('connecting to ' + host + ':' + port + ' ..', true);
   if (isRemote) {
+    self.print('connecting to ' + host + ':' + port + ' ..', true);
     attemptConnect();
   } else {
     this.child.stderr.once('data', function() {
+      self.print('connecting to ' + host + ':' + port + ' ..', true);
       setImmediate(attemptConnect);
     });
   }
