@@ -81,6 +81,17 @@ function ClientRequest(options, cb) {
     }
     if (host && !this.getHeader('host') && setHost) {
       var hostHeader = host;
+      var posColon = -1;
+
+      // For the Host header, ensure that IPv6 addresses are enclosed
+      // in square brackets, as defined by URI formatting
+      // https://tools.ietf.org/html/rfc3986#section-3.2.2
+      if (-1 !== (posColon = hostHeader.indexOf(':')) &&
+          -1 !== (posColon = hostHeader.indexOf(':', posColon)) &&
+          '[' !== hostHeader[0]) {
+        hostHeader = `[${hostHeader}]`;
+      }
+
       if (port && +port !== defaultPort) {
         hostHeader += ':' + port;
       }
@@ -115,8 +126,7 @@ function ClientRequest(options, cb) {
   if (self.socketPath) {
     self._last = true;
     self.shouldKeepAlive = false;
-    var conn = self.agent.createConnection({ path: self.socketPath });
-    self.onSocket(conn);
+    self.onSocket(self.agent.createConnection({ path: self.socketPath }));
   } else if (self.agent) {
     // If there is an agent we should default to Connection:keep-alive,
     // but only if the Agent will actually reuse the connection!
@@ -135,18 +145,19 @@ function ClientRequest(options, cb) {
     self._last = true;
     self.shouldKeepAlive = false;
     if (options.createConnection) {
-      var conn = options.createConnection(options);
+      self.onSocket(options.createConnection(options));
     } else {
       debug('CLIENT use net.createConnection', options);
-      var conn = net.createConnection(options);
+      self.onSocket(net.createConnection(options));
     }
-    self.onSocket(conn);
   }
 
   self._deferToConnect(null, null, function() {
     self._flush();
     self = null;
   });
+
+  this._ended = false;
 }
 
 util.inherits(ClientRequest, OutgoingMessage);
@@ -276,6 +287,13 @@ function socketErrorListener(err) {
   socket.destroy();
 }
 
+function freeSocketErrorListener(err) {
+  var socket = this;
+  debug('SOCKET ERROR on FREE socket:', err.message, err.stack);
+  socket.destroy();
+  socket.emit('agentRemove');
+}
+
 function socketOnEnd() {
   var socket = this;
   var req = this._httpMessage;
@@ -377,7 +395,7 @@ function parserOnIncomingClient(res, shouldKeepAlive) {
   // Responses to CONNECT request is handled as Upgrade.
   if (req.method === 'CONNECT') {
     res.upgrade = true;
-    return true; // skip body
+    return 2; // skip body, and the rest
   }
 
   // Responses to HEAD requests are crazy.
@@ -411,6 +429,7 @@ function parserOnIncomingClient(res, shouldKeepAlive) {
 
   // add our listener first, so that we guarantee socket cleanup
   res.on('end', responseOnEnd);
+  req.on('prefinish', requestOnPrefinish);
   var handled = req.emit('response', res);
 
   // If the user did not listen for the 'response' event, then they
@@ -423,9 +442,7 @@ function parserOnIncomingClient(res, shouldKeepAlive) {
 }
 
 // client
-function responseOnEnd() {
-  var res = this;
-  var req = res.req;
+function responseKeepAlive(res, req) {
   var socket = req.socket;
 
   if (!req.shouldKeepAlive) {
@@ -442,12 +459,33 @@ function responseOnEnd() {
     }
     socket.removeListener('close', socketCloseListener);
     socket.removeListener('error', socketErrorListener);
+    socket.once('error', freeSocketErrorListener);
     // Mark this socket as available, AFTER user-added end
     // handlers have a chance to run.
     process.nextTick(emitFreeNT, socket);
   }
 }
 
+function responseOnEnd() {
+  const res = this;
+  const req = this.req;
+
+  req._ended = true;
+  if (!req.shouldKeepAlive || req.finished)
+    responseKeepAlive(res, req);
+}
+
+function requestOnPrefinish() {
+  const req = this;
+  const res = this.res;
+
+  if (!req.shouldKeepAlive)
+    return;
+
+  if (req._ended)
+    responseKeepAlive(res, req);
+}
+
 function emitFreeNT(socket) {
   socket.emit('free');
 }
@@ -459,6 +497,7 @@ function tickOnSocket(req, socket) {
   parser.reinitialize(HTTPParser.RESPONSE);
   parser.socket = socket;
   parser.incoming = null;
+  parser.outgoing = req;
   req.parser = parser;
 
   socket.parser = parser;
@@ -476,6 +515,7 @@ function tickOnSocket(req, socket) {
   }
 
   parser.onIncoming = parserOnIncomingClient;
+  socket.removeListener('error', freeSocketErrorListener);
   socket.on('error', socketErrorListener);
   socket.on('data', socketOnData);
   socket.on('end', socketOnEnd);
@@ -561,10 +601,18 @@ ClientRequest.prototype.setTimeout = function(msecs, callback) {
 };
 
 ClientRequest.prototype.setNoDelay = function() {
-  this._deferToConnect('setNoDelay', arguments);
+  const argsLen = arguments.length;
+  const args = new Array(argsLen);
+  for (var i = 0; i < argsLen; i++)
+    args[i] = arguments[i];
+  this._deferToConnect('setNoDelay', args);
 };
 ClientRequest.prototype.setSocketKeepAlive = function() {
-  this._deferToConnect('setKeepAlive', arguments);
+  const argsLen = arguments.length;
+  const args = new Array(argsLen);
+  for (var i = 0; i < argsLen; i++)
+    args[i] = arguments[i];
+  this._deferToConnect('setKeepAlive', args);
 };
 
 ClientRequest.prototype.clearTimeout = function(cb) {
