@@ -91,6 +91,14 @@ Object.setPrototypeOf(Buffer, Uint8Array);
 /**
  * Creates a new filled Buffer instance.
  * alloc(size[, fill[, encoding]])
+ *
+ * Only pay attention to encoding if it's a string. This
+ * prevents accidentally sending in a number that would
+ * be interpreted as a start offset.
+ * Also, don't apply encoding if fill is a number.
+ *
+ * These comments are placed before the function to keep the text length
+ * down, to ensure that it remains inlineable by V8.
  **/
 Buffer.alloc = function(size, fill, encoding) {
   if (typeof size !== 'number')
@@ -98,14 +106,14 @@ Buffer.alloc = function(size, fill, encoding) {
   if (size <= 0)
     return createBuffer(size);
   if (fill !== undefined) {
-    // Only pay attention to encoding if it's a string. This
-    // prevents accidentally sending in a number that would
-    // be interpretted as a start offset.
-    // Also, don't apply encoding if fill is a number.
     if (typeof fill !== 'number' && typeof encoding === 'string')
       fill = Buffer.from(fill, encoding);
 
-    return createBuffer(size, true).fill(fill);
+    const buf = createBuffer(size, true);
+    // Buffer.prototype.fill does not support filling with other buffers in v4.
+    // Instead, call binding.fill directly.
+    binding.fill(buf, fill, 0, buf.length);
+    return buf;
   }
   return createBuffer(size);
 };
