@@ -40,13 +40,13 @@ function lookup6(address, callback) {
 
 function newHandle(type) {
   if (type == 'udp4') {
-    var handle = new UDP();
+    const handle = new UDP();
     handle.lookup = lookup4;
     return handle;
   }
 
   if (type == 'udp6') {
-    var handle = new UDP();
+    const handle = new UDP();
     handle.lookup = lookup6;
     handle.bind = handle.bind6;
     handle.send = handle.send6;
@@ -134,8 +134,9 @@ function replaceHandle(self, newHandle) {
   self._handle = newHandle;
 }
 
-Socket.prototype.bind = function(port /*, address, callback*/) {
+Socket.prototype.bind = function(port_ /*, address, callback*/) {
   var self = this;
+  let port = port_;
 
   self._healthCheck();
 
@@ -208,7 +209,7 @@ Socket.prototype.bind = function(port /*, address, callback*/) {
       if (!self._handle)
         return; // handle has been closed in the mean time
 
-      var err = self._handle.bind(ip, port || 0, flags);
+      const err = self._handle.bind(ip, port || 0, flags);
       if (err) {
         var ex = exceptionWithHostPort(err, 'bind', ip, port);
         self.emit('error', ex);
@@ -242,6 +243,32 @@ Socket.prototype.sendto = function(buffer,
 };
 
 
+function enqueue(self, toEnqueue) {
+  // If the send queue hasn't been initialized yet, do it, and install an
+  // event handler that flushes the send queue after binding is done.
+  if (!self._queue) {
+    self._queue = [];
+    self.once('listening', clearQueue);
+  }
+  self._queue.push(toEnqueue);
+  return;
+}
+
+
+function clearQueue() {
+  const queue = this._queue;
+  this._queue = undefined;
+
+  // Flush the send queue.
+  for (var i = 0; i < queue.length; i++)
+    queue[i]();
+}
+
+
+// valid combinations
+// send(buffer, offset, length, port, address, callback)
+// send(buffer, offset, length, port, address)
+// send(buffer, offset, length, port)
 Socket.prototype.send = function(buffer,
                                  offset,
                                  length,
@@ -289,18 +316,13 @@ Socket.prototype.send = function(buffer,
   // If the socket hasn't been bound yet, push the outbound packet onto the
   // send queue and send after binding is complete.
   if (self._bindState != BIND_STATE_BOUND) {
-    // If the send queue hasn't been initialized yet, do it, and install an
-    // event handler that flushes the send queue after binding is done.
-    if (!self._sendQueue) {
-      self._sendQueue = [];
-      self.once('listening', function() {
-        // Flush the send queue.
-        for (var i = 0; i < self._sendQueue.length; i++)
-          self.send.apply(self, self._sendQueue[i]);
-        self._sendQueue = undefined;
-      });
-    }
-    self._sendQueue.push([buffer, offset, length, port, address, callback]);
+    enqueue(self, self.send.bind(self,
+                                 buffer,
+                                 offset,
+                                 length,
+                                 port,
+                                 address,
+                                 callback));
     return;
   }
 
@@ -331,7 +353,7 @@ Socket.prototype.send = function(buffer,
                                   !!callback);
       if (err && callback) {
         // don't emit as error, dgram_legacy.js compatibility
-        var ex = exceptionWithHostPort(err, 'send', address, port);
+        const ex = exceptionWithHostPort(err, 'send', address, port);
         process.nextTick(callback, ex);
       }
     }
@@ -346,16 +368,20 @@ function afterSend(err) {
   this.callback(err, this.length);
 }
 
-
 Socket.prototype.close = function(callback) {
   if (typeof callback === 'function')
     this.on('close', callback);
+
+  if (this._queue) {
+    this._queue.push(this.close.bind(this));
+    return this;
+  }
+
   this._healthCheck();
   this._stopReceiving();
   this._handle.close();
   this._handle = null;
-  var self = this;
-  process.nextTick(socketCloseNT, self);
+  process.nextTick(socketCloseNT, this);
 
   return this;
 };
