@@ -90,10 +90,21 @@ const handleConversion = {
       return handle;
     },
 
-    postSend: function(handle) {
-      // Close the Socket handle after sending it
-      if (handle)
-        handle.close();
+    postSend: function(handle, target) {
+      // Store the handle after successfully sending it, so it can be closed
+      // when the NODE_HANDLE_ACK is received. If the handle could not be sent,
+      // just close it.
+      if (handle) {
+        if (target) {
+          // There can only be one _pendingHandle as passing handles are
+          // processed one at a time: handles are stored in _handleQueue while
+          // waiting for the NODE_HANDLE_ACK of the current passing handle.
+          assert(!target._pendingHandle);
+          target._pendingHandle = handle;
+        } else {
+          handle.close();
+        }
+      }
     },
 
     got: function(message, handle, emit) {
@@ -217,7 +228,7 @@ util.inherits(ChildProcess, EventEmitter);
 function flushStdio(subprocess) {
   if (subprocess.stdio == null) return;
   subprocess.stdio.forEach(function(stream, fd, stdio) {
-    if (!stream || !stream.readable)
+    if (!stream || !stream.readable || stream._readableState.readableListening)
       return;
     stream.resume();
   });
@@ -250,11 +261,11 @@ function getHandleWrapType(stream) {
 
 
 ChildProcess.prototype.spawn = function(options) {
-  var self = this,
-      ipc,
-      ipcFd,
-      // If no `stdio` option was given - use default
-      stdio = options.stdio || 'pipe';
+  const self = this;
+  var ipc;
+  var ipcFd;
+  // If no `stdio` option was given - use default
+  var stdio = options.stdio || 'pipe';
 
   stdio = _validateStdio(stdio, false);
 
@@ -396,6 +407,7 @@ ChildProcess.prototype.unref = function() {
 function setupChannel(target, channel) {
   target._channel = channel;
   target._handleQueue = null;
+  target._pendingHandle = null;
 
   const control = new class extends EventEmitter {
     constructor() {
@@ -461,6 +473,11 @@ function setupChannel(target, channel) {
   target.on('internalMessage', function(message, handle) {
     // Once acknowledged - continue sending handles.
     if (message.cmd === 'NODE_HANDLE_ACK') {
+      if (target._pendingHandle) {
+        target._pendingHandle.close();
+        target._pendingHandle = null;
+      }
+
       assert(Array.isArray(target._handleQueue));
       var queue = target._handleQueue;
       target._handleQueue = null;
@@ -587,13 +604,16 @@ function setupChannel(target, channel) {
     var err = channel.writeUtf8String(req, string, handle);
 
     if (err === 0) {
-      if (handle && !this._handleQueue)
-        this._handleQueue = [];
+      if (handle) {
+        if (!this._handleQueue)
+          this._handleQueue = [];
+        if (obj && obj.postSend)
+          obj.postSend(handle, target);
+      }
+
       req.oncomplete = function() {
         if (this.async === true)
           control.unref();
-        if (obj && obj.postSend)
-          obj.postSend(handle);
         if (typeof callback === 'function')
           callback(null);
       };
@@ -602,12 +622,18 @@ function setupChannel(target, channel) {
       } else {
         process.nextTick(function() { req.oncomplete(); });
       }
-    } else if (!swallowErrors) {
-      const ex = errnoException(err, 'write');
-      if (typeof callback === 'function') {
-        process.nextTick(callback, ex);
-      } else {
-        this.emit('error', ex);  // FIXME(bnoordhuis) Defer to next tick.
+    } else {
+      // Cleanup handle on error
+      if (obj && obj.postSend)
+        obj.postSend(handle);
+
+      if (!swallowErrors) {
+        const ex = errnoException(err, 'write');
+        if (typeof callback === 'function') {
+          process.nextTick(callback, ex);
+        } else {
+          this.emit('error', ex);  // FIXME(bnoordhuis) Defer to next tick.
+        }
       }
     }
 
@@ -648,6 +674,11 @@ function setupChannel(target, channel) {
     // This marks the fact that the channel is actually disconnected.
     this._channel = null;
 
+    if (this._pendingHandle) {
+      this._pendingHandle.close();
+      this._pendingHandle = null;
+    }
+
     var fired = false;
     function finish() {
       if (fired) return;
@@ -686,14 +717,16 @@ function handleMessage(target, message, handle) {
       message.cmd.slice(0, INTERNAL_PREFIX.length) === INTERNAL_PREFIX) {
     eventName = 'internalMessage';
   }
-  target.emit(eventName, message, handle);
+  process.nextTick(() => {
+    target.emit(eventName, message, handle);
+  });
 }
 
 function nop() { }
 
 function _validateStdio(stdio, sync) {
-  var ipc,
-      ipcFd;
+  var ipc;
+  var ipcFd;
 
   // Replace shortcut with an array
   if (typeof stdio === 'string') {
