@@ -59,9 +59,8 @@ exports.createServer = function(options, connectionListener) {
 // connect(path, [cb]);
 //
 exports.connect = exports.createConnection = function() {
-  const argsLen = arguments.length;
-  var args = new Array(argsLen);
-  for (var i = 0; i < argsLen; i++)
+  var args = new Array(arguments.length);
+  for (var i = 0; i < arguments.length; i++)
     args[i] = arguments[i];
   args = normalizeConnectArgs(args);
   debug('createConnection', args);
@@ -74,7 +73,9 @@ exports.connect = exports.createConnection = function() {
 function normalizeConnectArgs(args) {
   var options = {};
 
-  if (args[0] !== null && typeof args[0] === 'object') {
+  if (args.length === 0) {
+    return [options];
+  } else if (args[0] !== null && typeof args[0] === 'object') {
     // connect(options, [cb])
     options = args[0];
   } else if (isPipeName(args[0])) {
@@ -83,7 +84,7 @@ function normalizeConnectArgs(args) {
   } else {
     // connect(port, [host], [cb])
     options.port = args[0];
-    if (typeof args[1] === 'string') {
+    if (args.length > 1 && typeof args[1] === 'string') {
       options.host = args[1];
     }
   }
@@ -559,14 +560,16 @@ function onread(nread, buffer) {
 
   debug('EOF');
 
+  // push a null to signal the end of data.
+  // Do it before `maybeDestroy` for correct order of events:
+  // `end` -> `close`
+  self.push(null);
+
   if (self._readableState.length === 0) {
     self.readable = false;
     maybeDestroy(self);
   }
 
-  // push a null to signal the end of data.
-  self.push(null);
-
   // internal end event so that we know that the actual socket
   // is no longer readable, and we can start the shutdown
   // procedure. No need to wait for all the data to be consumed.
@@ -685,7 +688,6 @@ Socket.prototype._writeGeneric = function(writev, data, encoding, cb) {
   } else {
     var enc;
     if (data instanceof Buffer) {
-      req.buffer = data;  // Keep reference alive.
       enc = 'buffer';
     } else {
       enc = encoding;
@@ -876,9 +878,8 @@ Socket.prototype.connect = function(options, cb) {
     // Old API:
     // connect(port, [host], [cb])
     // connect(path, [cb]);
-    const argsLen = arguments.length;
-    var args = new Array(argsLen);
-    for (var i = 0; i < argsLen; i++)
+    var args = new Array(arguments.length);
+    for (var i = 0; i < arguments.length; i++)
       args[i] = arguments[i];
     args = normalizeConnectArgs(args);
     return Socket.prototype.connect.apply(this, args);
@@ -1155,8 +1156,7 @@ function createServerHandle(address, port, addressType, fd) {
   if (typeof fd === 'number' && fd >= 0) {
     try {
       handle = createHandle(fd);
-    }
-    catch (e) {
+    } catch (e) {
       // Not a fd we can listen on.  This will trigger an error.
       debug('listen invalid fd=' + fd + ': ' + e.message);
       return uv.UV_EINVAL;
@@ -1569,14 +1569,10 @@ Server.prototype.unref = function() {
 exports.isIP = cares.isIP;
 
 
-exports.isIPv4 = function(input) {
-  return cares.isIPv4(input);
-};
+exports.isIPv4 = cares.isIPv4;
 
 
-exports.isIPv6 = function(input) {
-  return cares.isIPv6(input);
-};
+exports.isIPv6 = cares.isIPv6;
 
 
 if (process.platform === 'win32') {
