@@ -9,6 +9,7 @@
 const kHistorySize = 30;
 
 const util = require('util');
+const debug = util.debuglog('readline');
 const internalUtil = require('internal/util');
 const inherits = util.inherits;
 const Buffer = require('buffer').Buffer;
@@ -35,6 +36,7 @@ function Interface(input, output, completer, terminal) {
   }
 
   this._sawReturn = false;
+  this.isCompletionEnabled = true;
 
   EventEmitter.call(this);
   var historySize;
@@ -121,7 +123,7 @@ function Interface(input, output, completer, terminal) {
 
   } else {
 
-    exports.emitKeypressEvents(input);
+    exports.emitKeypressEvents(input, this);
 
     // input usually refers to stdin
     input.on('keypress', onkeypress);
@@ -329,7 +331,7 @@ Interface.prototype._normalWrite = function(b) {
     this._line_buffer = null;
   }
   if (newPartContainsEnding) {
-    this._sawReturn = /\r$/.test(string);
+    this._sawReturn = string.endsWith('\r');
 
     // got one or more newlines; process into "line" events
     var lines = string.split(lineEnding);
@@ -375,12 +377,12 @@ Interface.prototype._tabComplete = function() {
     self.resume();
 
     if (err) {
-      // XXX Log it somewhere?
+      debug('tab completion error %j', err);
       return;
     }
 
-    var completions = rv[0],
-        completeOn = rv[1];  // the text that was completed
+    const completions = rv[0];
+    const completeOn = rv[1];  // the text that was completed
     if (completions && completions.length) {
       // Apply/show completions.
       if (completions.length === 1) {
@@ -867,7 +869,7 @@ Interface.prototype._ttyWrite = function(s, key) {
 
       case 'tab':
         // If tab completion enabled, do that...
-        if (typeof this.completer === 'function') {
+        if (typeof this.completer === 'function' && this.isCompletionEnabled) {
           this._tabComplete();
           break;
         }
@@ -901,7 +903,10 @@ exports.Interface = Interface;
 const KEYPRESS_DECODER = Symbol('keypress-decoder');
 const ESCAPE_DECODER = Symbol('escape-decoder');
 
-function emitKeypressEvents(stream) {
+// GNU readline library - keyseq-timeout is 500ms (default)
+const ESCAPE_CODE_TIMEOUT = 500;
+
+function emitKeypressEvents(stream, iface) {
   if (stream[KEYPRESS_DECODER]) return;
   var StringDecoder = require('string_decoder').StringDecoder; // lazy load
   stream[KEYPRESS_DECODER] = new StringDecoder('utf8');
@@ -909,19 +914,36 @@ function emitKeypressEvents(stream) {
   stream[ESCAPE_DECODER] = emitKeys(stream);
   stream[ESCAPE_DECODER].next();
 
+  const escapeCodeTimeout = () => stream[ESCAPE_DECODER].next('');
+  let timeoutId;
+
   function onData(b) {
     if (stream.listenerCount('keypress') > 0) {
       var r = stream[KEYPRESS_DECODER].write(b);
       if (r) {
+        clearTimeout(timeoutId);
+
         for (var i = 0; i < r.length; i++) {
+          if (r[i] === '\t' && typeof r[i + 1] === 'string' && iface) {
+            iface.isCompletionEnabled = false;
+          }
+
           try {
             stream[ESCAPE_DECODER].next(r[i]);
+            // Escape letter at the tail position
+            if (r[i] === '\x1b' && i + 1 === r.length) {
+              timeoutId = setTimeout(escapeCodeTimeout, ESCAPE_CODE_TIMEOUT);
+            }
           } catch (err) {
             // if the generator throws (it could happen in the `keypress`
             // event), we need to restart it.
             stream[ESCAPE_DECODER] = emitKeys(stream);
             stream[ESCAPE_DECODER].next();
             throw err;
+          } finally {
+            if (iface) {
+              iface.isCompletionEnabled = true;
+            }
           }
         }
       }
@@ -976,6 +998,7 @@ exports.emitKeypressEvents = emitKeypressEvents;
 */
 
 // Regexes used for ansi escape code splitting
+// eslint-disable-next-line no-control-regex
 const metaKeyCodeReAnywhere = /(?:\x1b)([a-zA-Z0-9])/;
 const functionKeyCodeReAnywhere = new RegExp('(?:\x1b+)(O|N|\\[|\\[\\[)(?:' + [
   '(\\d+)(?:;(\\d+))?([~^$])',
@@ -1241,11 +1264,15 @@ function* emitKeys(stream) {
       key.name = ch.toLowerCase();
       key.shift = /^[A-Z]$/.test(ch);
       key.meta = escaped;
+    } else if (escaped) {
+      // Escape sequence timeout
+      key.name = ch.length ? undefined : 'escape';
+      key.meta = true;
     }
 
     key.sequence = s;
 
-    if (key.name !== undefined) {
+    if (s.length !== 0 && (key.name !== undefined || escaped)) {
       /* Named character or sequence */
       stream.emit('keypress', escaped ? undefined : s, key);
     } else if (s.length === 1) {
