@@ -38,6 +38,16 @@ const debug = util.debuglog('repl');
 const parentModule = module;
 const replMap = new WeakMap();
 
+const GLOBAL_OBJECT_PROPERTIES = ['NaN', 'Infinity', 'undefined',
+  'eval', 'parseInt', 'parseFloat', 'isNaN', 'isFinite', 'decodeURI',
+  'decodeURIComponent', 'encodeURI', 'encodeURIComponent',
+  'Object', 'Function', 'Array', 'String', 'Boolean', 'Number',
+  'Date', 'RegExp', 'Error', 'EvalError', 'RangeError',
+  'ReferenceError', 'SyntaxError', 'TypeError', 'URIError',
+  'Math', 'JSON'];
+const GLOBAL_OBJECT_PROPERTY_MAP = {};
+GLOBAL_OBJECT_PROPERTIES.forEach((p) => GLOBAL_OBJECT_PROPERTY_MAP[p] = p);
+
 try {
   // hack for require.resolve("./relative") to work properly.
   module.filename = path.resolve('repl');
@@ -70,8 +80,8 @@ exports._builtinLibs = ['assert', 'buffer', 'child_process', 'cluster',
   'string_decoder', 'tls', 'tty', 'url', 'util', 'v8', 'vm', 'zlib'];
 
 
-const BLOCK_SCOPED_ERROR = 'Block-scoped declarations (let, ' +
-    'const, function, class) not yet supported outside strict mode';
+const BLOCK_SCOPED_ERROR = 'Block-scoped declarations (let, const, function, ' +
+                           'class) not yet supported outside strict mode';
 
 
 class LineParser {
@@ -84,6 +94,7 @@ class LineParser {
     this._literal = null;
     this.shouldFail = false;
     this.blockComment = false;
+    this.regExpLiteral = false;
   }
 
   parseLine(line) {
@@ -99,6 +110,11 @@ class LineParser {
       }
 
       if (!this._literal) {
+        if (this.regExpLiteral && current === '/') {
+          this.regExpLiteral = false;
+          previous = null;
+          continue;
+        }
         if (previous === '*' && current === '/') {
           if (this.blockComment) {
             this.blockComment = false;
@@ -115,13 +131,17 @@ class LineParser {
           break;
         }
 
-        if (previous === '/' && current === '*') {
-          this.blockComment = true;
+        if (previous === '/') {
+          if (current === '*') {
+            this.blockComment = true;
+          } else {
+            this.regExpLiteral = true;
+          }
           previous = null;
         }
       }
 
-      if (this.blockComment) continue;
+      if (this.blockComment || this.regExpLiteral) continue;
 
       if (current === this._literal) {
         this._literal = null;
@@ -220,7 +240,7 @@ function REPLServer(prompt,
             (self.replMode === exports.REPL_MODE_STRICT || retry)) {
           // "void 0" keeps the repl from returning "use strict" as the
           // result value for let/const statements.
-          code = `'use strict'; void 0; ${code}`;
+          code = `'use strict'; void 0;\n${code}`;
         }
         var script = vm.createScript(code, {
           filename: file,
@@ -266,7 +286,7 @@ function REPLServer(prompt,
 
     // After executing the current expression, store the values of RegExp
     // predefined properties back in `savedRegExMatches`
-    for (let idx = 1; idx < savedRegExMatches.length; idx += 1) {
+    for (var idx = 1; idx < savedRegExMatches.length; idx += 1) {
       savedRegExMatches[idx] = RegExp[`$${idx}`];
     }
 
@@ -279,6 +299,10 @@ function REPLServer(prompt,
     debug('domain error');
     const top = replMap.get(self);
     internalUtil.decorateErrorStack(e);
+    if (e.stack && self.replMode === exports.REPL_MODE_STRICT) {
+      e.stack = e.stack.replace(/(\s+at\s+repl:)(\d+)/,
+                                (_, pre, line) => pre + (line - 1));
+    }
     top.outputStream.write((e.stack || e) + '\n');
     top.lineParser.reset();
     top.bufferedCommand = '';
@@ -423,7 +447,7 @@ function REPLServer(prompt,
       debug('finish', e, ret);
       self.memory(cmd);
 
-      if (e && !self.bufferedCommand && cmd.trim().match(/^npm /)) {
+      if (e && !self.bufferedCommand && cmd.trim().startsWith('npm ')) {
         self.outputStream.write('npm should be run outside of the ' +
                                 'node repl, in your normal shell.\n' +
                                 '(Press Control-D to exit.)\n');
@@ -506,10 +530,20 @@ REPLServer.prototype.createContext = function() {
     context = global;
   } else {
     context = vm.createContext();
-    for (var i in global) context[i] = global[i];
-    context.console = new Console(this.outputStream);
     context.global = context;
-    context.global.global = context;
+    const _console = new Console(this.outputStream);
+    Object.defineProperty(context, 'console', {
+      configurable: true,
+      enumerable: true,
+      get: () => _console
+    });
+    Object.getOwnPropertyNames(global).filter((name) => {
+      if (name === 'console' || name === 'global') return false;
+      return GLOBAL_OBJECT_PROPERTY_MAP[name] === undefined;
+    }).forEach((name) => {
+      Object.defineProperty(context, name,
+        Object.getOwnPropertyDescriptor(global, name));
+    });
   }
 
   const module = new Module('<repl>');
@@ -588,7 +622,7 @@ ArrayStream.prototype.writable = true;
 ArrayStream.prototype.resume = function() {};
 ArrayStream.prototype.write = function() {};
 
-const requireRE = /\brequire\s*\(['"](([\w\.\/-]+\/)?([\w\.\/-]*))/;
+const requireRE = /\brequire\s*\(['"](([\w./-]+\/)?([\w./-]*))/;
 const simpleExpressionRE =
     /(([a-zA-Z_$](?:\w|\$)*)\.)*([a-zA-Z_$](?:\w|\$)*)\.?$/;
 
@@ -641,15 +675,15 @@ REPLServer.prototype.complete = function(line, callback) {
   // list of completion lists, one for each inheritance "level"
   var completionGroups = [];
 
-  var completeOn, match, filter, i, group, c;
+  var completeOn, i, group, c;
 
   // REPL commands (e.g. ".break").
   var match = null;
-  match = line.match(/^\s*(\.\w*)$/);
+  match = line.match(/^\s*\.(\w*)$/);
   if (match) {
     completionGroups.push(Object.keys(this.commands));
     completeOn = match[1];
-    if (match[1].length > 1) {
+    if (match[1].length) {
       filter = match[1];
     }
 
@@ -836,7 +870,7 @@ REPLServer.prototype.complete = function(line, callback) {
       var newCompletionGroups = [];
       for (i = 0; i < completionGroups.length; i++) {
         group = completionGroups[i].filter(function(elem) {
-          return elem.indexOf(filter) == 0;
+          return elem.indexOf(filter) === 0;
         });
         if (group.length) {
           newCompletionGroups.push(group);
@@ -975,13 +1009,7 @@ REPLServer.prototype.memory = function memory(cmd) {
 function addStandardGlobals(completionGroups, filter) {
   // Global object properties
   // (http://www.ecma-international.org/publications/standards/Ecma-262.htm)
-  completionGroups.push(['NaN', 'Infinity', 'undefined',
-    'eval', 'parseInt', 'parseFloat', 'isNaN', 'isFinite', 'decodeURI',
-    'decodeURIComponent', 'encodeURI', 'encodeURIComponent',
-    'Object', 'Function', 'Array', 'String', 'Boolean', 'Number',
-    'Date', 'RegExp', 'Error', 'EvalError', 'RangeError',
-    'ReferenceError', 'SyntaxError', 'TypeError', 'URIError',
-    'Math', 'JSON']);
+  completionGroups.push(GLOBAL_OBJECT_PROPERTIES);
   // Common keywords. Exclude for completion on the empty string, b/c
   // they just get in the way.
   if (filter) {
@@ -1096,9 +1124,9 @@ function regexpEscape(s) {
  * @return {String} The converted command.
  */
 REPLServer.prototype.convertToContext = function(cmd) {
-  const scopeVar = /^\s*var\s*([_\w\$]+)(.*)$/m;
-  const scopeFunc = /^\s*function\s*([_\w\$]+)/;
-  var self = this, matches;
+  const scopeVar = /^\s*var\s*([\w$]+)(.*)$/m;
+  const scopeFunc = /^\s*function\s*([\w$]+)/;
+  var matches;
 
   // Replaces: var foo = "bar";  with: self.context.foo = bar;
   matches = scopeVar.exec(cmd);
@@ -1107,9 +1135,9 @@ REPLServer.prototype.convertToContext = function(cmd) {
   }
 
   // Replaces: function foo() {};  with: foo = function foo() {};
-  matches = scopeFunc.exec(self.bufferedCommand);
+  matches = scopeFunc.exec(this.bufferedCommand);
   if (matches && matches.length === 2) {
-    return matches[1] + ' = ' + self.bufferedCommand;
+    return matches[1] + ' = ' + this.bufferedCommand;
   }
 
   return cmd;
