@@ -1,7 +1,7 @@
 'use strict';
 
 const Timer = process.binding('timer_wrap').Timer;
-const L = require('_linklist');
+const L = require('_linklist'); // eslint-disable-line no-restricted-modules
 const assert = require('assert').ok;
 const util = require('util');
 const debug = util.debuglog('timer');
@@ -272,14 +272,18 @@ exports.setInterval = function(callback, repeat) {
   return timer;
 
   function wrapper() {
+    // If _repeat was overriden we are effectively canceled. Bail.
+    if (typeof timer._repeat !== 'function')
+      return;
+
     timer._repeat();
 
-    // Timer might be closed - no point in restarting it
-    if (!timer._repeat)
+    // Do not re-arm unenroll'd or closed timers.
+    if (timer._idleTimeout === -1)
       return;
 
     // If timer is unref'd (or was - it's permanently removed from the list.)
-    if (this._handle) {
+    if (this._handle && timer instanceof Timeout) {
       this._handle.start(repeat, 0);
     } else {
       timer._idleTimeout = repeat;
@@ -309,16 +313,24 @@ const Timeout = function(after) {
 
 
 function unrefdHandle() {
-  this.owner._onTimeout();
-  if (!this.owner._repeat)
+  // Don't attempt to call the callback if it is not a function.
+  if (typeof this.owner._onTimeout === 'function') {
+    this.owner._onTimeout();
+  }
+
+  // Make sure we clean up if the callback is no longer a function
+  // even if the timer is an interval.
+  if (!this.owner._repeat
+      || typeof this.owner._onTimeout !== 'function') {
     this.owner.close();
+  }
 }
 
 
 Timeout.prototype.unref = function() {
   if (this._handle) {
     this._handle.unref();
-  } else if (typeof(this._onTimeout) === 'function') {
+  } else if (typeof this._onTimeout === 'function') {
     var now = Timer.now();
     if (!this._idleStart) this._idleStart = now;
     var delay = this._idleStart + this._idleTimeout - now;
@@ -351,6 +363,7 @@ Timeout.prototype.ref = function() {
 Timeout.prototype.close = function() {
   this._onTimeout = null;
   if (this._handle) {
+    this._idleTimeout = -1;
     this._handle[kOnTimeout] = null;
     this._handle.close();
   } else {
