@@ -59,6 +59,13 @@ function WriteStream(fd) {
     writable: true
   });
 
+  // Prevents interleaved stdout/stderr output in OS X terminals.
+  // As noted in the following reference, local TTYs tend to be quite fast and
+  // this behaviour has become expected due historical functionality on OS X,
+  // even though it was originally intended to change in v1.0.2 (Libuv 1.2.1).
+  // Ref: https://github.com/nodejs/node/pull/1771#issuecomment-119351671
+  if (process.platform === 'darwin') this._handle.setBlocking(true);
+
   var winSize = [];
   var err = this._handle.getWindowSize(winSize);
   if (!err) {
