@@ -9,39 +9,73 @@ const isError = internalUtil.isError;
 
 var Debug;
 
-const formatRegExp = /%[sdj%]/g;
+function tryStringify(arg) {
+  try {
+    return JSON.stringify(arg);
+  } catch (_) {
+    return '[Circular]';
+  }
+}
+
 exports.format = function(f) {
   if (typeof f !== 'string') {
-    var objects = [];
-    for (var i = 0; i < arguments.length; i++) {
-      objects.push(inspect(arguments[i]));
+    const objects = new Array(arguments.length);
+    for (var index = 0; index < arguments.length; index++) {
+      objects[index] = inspect(arguments[index]);
     }
     return objects.join(' ');
   }
 
-  if (arguments.length === 1) return f;
+  var argLen = arguments.length;
 
-  var i = 1;
-  var args = arguments;
-  var len = args.length;
-  var str = String(f).replace(formatRegExp, function(x) {
-    if (x === '%%') return '%';
-    if (i >= len) return x;
-    switch (x) {
-      case '%s': return String(args[i++]);
-      case '%d': return Number(args[i++]);
-      case '%j':
-        try {
-          return JSON.stringify(args[i++]);
-        } catch (_) {
-          return '[Circular]';
-        }
-        // falls through
-      default:
-        return x;
+  if (argLen === 1) return f;
+
+  var str = '';
+  var a = 1;
+  var lastPos = 0;
+  for (var i = 0; i < f.length;) {
+    if (f.charCodeAt(i) === 37/*'%'*/ && i + 1 < f.length) {
+      switch (f.charCodeAt(i + 1)) {
+        case 100: // 'd'
+          if (a >= argLen)
+            break;
+          if (lastPos < i)
+            str += f.slice(lastPos, i);
+          str += Number(arguments[a++]);
+          lastPos = i = i + 2;
+          continue;
+        case 106: // 'j'
+          if (a >= argLen)
+            break;
+          if (lastPos < i)
+            str += f.slice(lastPos, i);
+          str += tryStringify(arguments[a++]);
+          lastPos = i = i + 2;
+          continue;
+        case 115: // 's'
+          if (a >= argLen)
+            break;
+          if (lastPos < i)
+            str += f.slice(lastPos, i);
+          str += String(arguments[a++]);
+          lastPos = i = i + 2;
+          continue;
+        case 37: // '%'
+          if (lastPos < i)
+            str += f.slice(lastPos, i);
+          str += '%';
+          lastPos = i = i + 2;
+          continue;
+      }
     }
-  });
-  for (var x = args[i]; i < len; x = args[++i]) {
+    ++i;
+  }
+  if (lastPos === 0)
+    str = f;
+  else if (lastPos < f.length)
+    str += f.slice(lastPos);
+  while (a < argLen) {
+    const x = arguments[a++];
     if (x === null || (typeof x !== 'object' && typeof x !== 'symbol')) {
       str += ' ' + x;
     } else {
@@ -62,7 +96,7 @@ exports.debuglog = function(set) {
     debugEnviron = process.env.NODE_DEBUG || '';
   set = set.toUpperCase();
   if (!debugs[set]) {
-    if (new RegExp('\\b' + set + '\\b', 'i').test(debugEnviron)) {
+    if (new RegExp(`\\b${set}\\b`, 'i').test(debugEnviron)) {
       var pid = process.pid;
       debugs[set] = function() {
         var msg = exports.format.apply(exports, arguments);
@@ -77,7 +111,7 @@ exports.debuglog = function(set) {
 
 
 /**
- * Echos the value of a value. Trys to print the value out
+ * Echos the value of a value. Tries to print the value out
  * in the best way possible given the different types.
  *
  * @param {Object} obj The object to print out.
@@ -113,19 +147,19 @@ exports.inspect = inspect;
 
 // http://en.wikipedia.org/wiki/ANSI_escape_code#graphics
 inspect.colors = {
-  'bold' : [1, 22],
-  'italic' : [3, 23],
-  'underline' : [4, 24],
-  'inverse' : [7, 27],
-  'white' : [37, 39],
-  'grey' : [90, 39],
-  'black' : [30, 39],
-  'blue' : [34, 39],
-  'cyan' : [36, 39],
-  'green' : [32, 39],
-  'magenta' : [35, 39],
-  'red' : [31, 39],
-  'yellow' : [33, 39]
+  'bold': [1, 22],
+  'italic': [3, 23],
+  'underline': [4, 24],
+  'inverse': [7, 27],
+  'white': [37, 39],
+  'grey': [90, 39],
+  'black': [30, 39],
+  'blue': [34, 39],
+  'cyan': [36, 39],
+  'green': [32, 39],
+  'magenta': [35, 39],
+  'red': [31, 39],
+  'yellow': [33, 39]
 };
 
 // Don't use 'blue' not visible on cmd.exe
@@ -147,8 +181,8 @@ function stylizeWithColor(str, styleType) {
   var style = inspect.styles[styleType];
 
   if (style) {
-    return '\u001b[' + inspect.colors[style][0] + 'm' + str +
-           '\u001b[' + inspect.colors[style][1] + 'm';
+    return `\u001b[${inspect.colors[style][0]}m${str}` +
+           `\u001b[${inspect.colors[style][1]}m`;
   } else {
     return str;
   }
@@ -254,7 +288,7 @@ function formatValue(ctx, value, recurseTimes) {
 
   if (typeof raw === 'string') {
     // for boxed Strings, we have to remove the 0-n indexed entries,
-    // since they just noisey up the output and are redundant
+    // since they just noisy up the output and are redundant
     keys = keys.filter(function(key) {
       return !(key >= 0 && key < raw.length);
     });
@@ -263,8 +297,8 @@ function formatValue(ctx, value, recurseTimes) {
   // Some type of object without properties can be shortcutted.
   if (keys.length === 0) {
     if (typeof value === 'function') {
-      var name = value.name ? ': ' + value.name : '';
-      return ctx.stylize('[Function' + name + ']', 'special');
+      return ctx.stylize(`[Function${value.name ? `: ${value.name}` : ''}]`,
+        'special');
     }
     if (isRegExp(value)) {
       return ctx.stylize(RegExp.prototype.toString.call(value), 'regexp');
@@ -278,15 +312,19 @@ function formatValue(ctx, value, recurseTimes) {
     // now check the `raw` value to handle boxed primitives
     if (typeof raw === 'string') {
       formatted = formatPrimitiveNoColor(ctx, raw);
-      return ctx.stylize('[String: ' + formatted + ']', 'string');
+      return ctx.stylize(`[String: ${formatted}]`, 'string');
+    }
+    if (typeof raw === 'symbol') {
+      formatted = formatPrimitiveNoColor(ctx, raw);
+      return ctx.stylize(`[Symbol: ${formatted}]`, 'symbol');
     }
     if (typeof raw === 'number') {
       formatted = formatPrimitiveNoColor(ctx, raw);
-      return ctx.stylize('[Number: ' + formatted + ']', 'number');
+      return ctx.stylize(`[Number: ${formatted}]`, 'number');
     }
     if (typeof raw === 'boolean') {
       formatted = formatPrimitiveNoColor(ctx, raw);
-      return ctx.stylize('[Boolean: ' + formatted + ']', 'boolean');
+      return ctx.stylize(`[Boolean: ${formatted}]`, 'boolean');
     }
   }
 
@@ -352,8 +390,7 @@ function formatValue(ctx, value, recurseTimes) {
 
   // Make functions say that they are functions
   if (typeof value === 'function') {
-    var n = value.name ? ': ' + value.name : '';
-    base = ' [Function' + n + ']';
+    base = ` [Function${value.name ? `: ${value.name}` : ''}]`;
   }
 
   // Make RegExps say that they are RegExps
@@ -374,24 +411,24 @@ function formatValue(ctx, value, recurseTimes) {
   // Make boxed primitive Strings look like such
   if (typeof raw === 'string') {
     formatted = formatPrimitiveNoColor(ctx, raw);
-    base = ' ' + '[String: ' + formatted + ']';
+    base = ` [String: ${formatted}]`;
   }
 
   // Make boxed primitive Numbers look like such
   if (typeof raw === 'number') {
     formatted = formatPrimitiveNoColor(ctx, raw);
-    base = ' ' + '[Number: ' + formatted + ']';
+    base = ` [Number: ${formatted}]`;
   }
 
   // Make boxed primitive Booleans look like such
   if (typeof raw === 'boolean') {
     formatted = formatPrimitiveNoColor(ctx, raw);
-    base = ' ' + '[Boolean: ' + formatted + ']';
+    base = ` [Boolean: ${formatted}]`;
   }
 
   // Add constructor name if available
   if (base === '' && constructor)
-    braces[0] = constructor.name + ' ' + braces[0];
+    braces[0] = `${constructor.name} ${braces[0]}`;
 
   if (empty === true) {
     return braces[0] + base + braces[1];
@@ -426,9 +463,12 @@ function formatPrimitive(ctx, value) {
   var type = typeof value;
 
   if (type === 'string') {
-    var simple = '\'' + JSON.stringify(value).replace(/^"|"$/g, '')
-        .replace(/'/g, "\\'")
-        .replace(/\\"/g, '"') + '\'';
+    var simple = '\'' +
+                 JSON.stringify(value)
+                     .replace(/^"|"$/g, '')
+                     .replace(/'/g, "\\'")
+                     .replace(/\\"/g, '"') +
+                 '\'';
     return ctx.stylize(simple, 'string');
   }
   if (type === 'number') {
@@ -456,7 +496,7 @@ function formatPrimitiveNoColor(ctx, value) {
 
 
 function formatError(value) {
-  return '[' + Error.prototype.toString.call(value) + ']';
+  return `[${Error.prototype.toString.call(value)}]`;
 }
 
 
@@ -568,9 +608,9 @@ function formatProperty(ctx, value, recurseTimes, visibleKeys, key, array) {
   }
   if (!hasOwnProperty(visibleKeys, key)) {
     if (typeof key === 'symbol') {
-      name = '[' + ctx.stylize(key.toString(), 'symbol') + ']';
+      name = `[${ctx.stylize(key.toString(), 'symbol')}]`;
     } else {
-      name = '[' + key + ']';
+      name = `[${key}]`;
     }
   }
   if (!str) {
@@ -608,7 +648,7 @@ function formatProperty(ctx, value, recurseTimes, visibleKeys, key, array) {
     }
   }
 
-  return name + ': ' + str;
+  return `${name}: ${str}`;
 }
 
 
@@ -623,13 +663,10 @@ function reduceToSingleString(output, base, braces) {
            // we need to force the first item to be on the next line or the
            // items will not line up correctly.
            (base === '' && braces[0].length === 1 ? '' : base + '\n ') +
-           ' ' +
-           output.join(',\n  ') +
-           ' ' +
-           braces[1];
+           ` ${output.join(',\n  ')} ${braces[1]}`;
   }
 
-  return braces[0] + base + ' ' + output.join(', ') + ' ' + braces[1];
+  return `${braces[0]}${base} ${output.join(', ')} ${braces[1]}`;
 }
 
 
@@ -813,7 +850,7 @@ exports.puts = internalUtil.deprecate(function() {
 
 
 exports.debug = internalUtil.deprecate(function(x) {
-  process.stderr.write('DEBUG: ' + x + '\n');
+  process.stderr.write(`DEBUG: ${x}\n`);
 }, 'util.debug is deprecated. Use console.error instead.');
 
 
@@ -864,7 +901,7 @@ exports.pump = internalUtil.deprecate(function(readStream, writeStream, cb) {
 
 exports._errnoException = function(err, syscall, original) {
   var errname = uv.errname(err);
-  var message = syscall + ' ' + errname;
+  var message = `${syscall} ${errname}`;
   if (original)
     message += ' ' + original;
   var e = new Error(message);
@@ -882,13 +919,13 @@ exports._exceptionWithHostPort = function(err,
                                           additional) {
   var details;
   if (port && port > 0) {
-    details = address + ':' + port;
+    details = `${address}:${port}`;
   } else {
     details = address;
   }
 
   if (additional) {
-    details += ' - Local (' + additional + ')';
+    details += ` - Local (${additional})`;
   }
   var ex = exports._errnoException(err, syscall, details);
   ex.address = address;
