@@ -12,12 +12,14 @@
 
   function startup() {
     var EventEmitter = NativeModule.require('events');
+    process._eventsCount = 0;
 
-    process.__proto__ = Object.create(EventEmitter.prototype, {
+    Object.setPrototypeOf(process, Object.create(EventEmitter.prototype, {
       constructor: {
         value: process.constructor
       }
-    });
+    }));
+
     EventEmitter.call(process);
 
     process.EventEmitter = EventEmitter; // process.EventEmitter is deprecated
@@ -61,13 +63,14 @@
 
     } else if (process.argv[1] == 'debug') {
       // Start the debugger agent
-      var d = NativeModule.require('_debugger');
-      d.start();
+      NativeModule.require('_debugger').start();
 
     } else if (process.argv[1] == '--debug-agent') {
       // Start the debugger agent
-      var d = NativeModule.require('_debug_agent');
-      d.start();
+      NativeModule.require('_debug_agent').start();
+
+    } else if (process.profProcess) {
+      NativeModule.require('internal/v8_prof_processor');
 
     } else {
       // There is user code to be run
@@ -137,8 +140,6 @@
         }
 
       } else {
-        var Module = NativeModule.require('module');
-
         // If -i or --interactive were passed, or stdin is a TTY.
         if (process._forceRepl || NativeModule.require('tty').isatty(0)) {
           // REPL
@@ -265,6 +266,22 @@
     process.config = JSON.parse(config, function(key, value) {
       if (value === 'true') return true;
       if (value === 'false') return false;
+
+      // Intl.v8BreakIterator() would crash w/ fatal error, so throw instead.
+      if (value.icu_small &&
+          global.hasOwnProperty('Intl') &&
+          Intl.hasOwnProperty('v8BreakIterator') &&
+          !process.icu_data_dir) {
+        const des = Object.getOwnPropertyDescriptor(Intl, 'v8BreakIterator');
+        des.value = function v8BreakIterator() {
+          throw new Error('v8BreakIterator: full ICU data not installed. ' +
+                          'See https://github.com/nodejs/node/wiki/Intl');
+        };
+        Object.defineProperty(Intl, 'v8BreakIterator', des);
+      }
+      // Don’t let icu_data_dir leak through.
+      delete process.icu_data_dir;
+
       return value;
     });
   };
@@ -555,7 +572,7 @@
       // getcwd(3) can fail if the current working directory has been deleted.
       // Fall back to the directory name of the (absolute) executable path.
       // It's not really correct but what are the alternatives?
-      var cwd = path.dirname(process.execPath);
+      cwd = path.dirname(process.execPath);
     }
 
     var module = new Module(name);
@@ -563,19 +580,19 @@
     module.paths = Module._nodeModulePaths(cwd);
     var script = process._eval;
     var body = script;
-    script = 'global.__filename = ' + JSON.stringify(name) + ';\n' +
+    script = `global.__filename = ${JSON.stringify(name)};\n` +
              'global.exports = exports;\n' +
              'global.module = module;\n' +
              'global.__dirname = __dirname;\n' +
              'global.require = require;\n' +
              'return require("vm").runInThisContext(' +
-             JSON.stringify(body) + ', { filename: ' +
-             JSON.stringify(name) + ' });\n';
+             `${JSON.stringify(body)}, { filename: ` +
+             `${JSON.stringify(name)} });\n`;
     // Defer evaluation for a tick.  This is a workaround for deferred
     // events not firing when evaluating scripts from the command line,
     // see https://github.com/nodejs/node/issues/1600.
     process.nextTick(function() {
-      var result = module._compile(script, name + '-wrapper');
+      var result = module._compile(script, `${name}-wrapper`);
       if (process._print_eval) console.log(result);
     });
   }
@@ -767,7 +784,7 @@
             sig.slice(0, 3) === 'SIG') {
           err = process._kill(pid, startup.lazyConstants()[sig]);
         } else {
-          throw new Error('Unknown signal: ' + sig);
+          throw new Error(`Unknown signal: ${sig}`);
         }
       }
 
@@ -786,7 +803,8 @@
     var signalWraps = {};
 
     function isSignal(event) {
-      return event.slice(0, 3) === 'SIG' &&
+      return typeof event === 'string' &&
+             event.slice(0, 3) === 'SIG' &&
              startup.lazyConstants().hasOwnProperty(event);
     }
 
@@ -871,10 +889,11 @@
   }
 
   function NativeModule(id) {
-    this.filename = id + '.js';
+    this.filename = `${id}.js`;
     this.id = id;
     this.exports = {};
     this.loaded = false;
+    this.loading = true;
   }
 
   NativeModule._source = process.binding('natives');
@@ -886,15 +905,15 @@
     }
 
     var cached = NativeModule.getCached(id);
-    if (cached) {
+    if (cached && (cached.loaded || cached.loading)) {
       return cached.exports;
     }
 
     if (!NativeModule.exists(id)) {
-      throw new Error('No such native module ' + id);
+      throw new Error(`No such native module ${id}`);
     }
 
-    process.moduleLoadList.push('NativeModule ' + id);
+    process.moduleLoadList.push(`NativeModule ${id}`);
 
     var nativeModule = new NativeModule(id);
 
@@ -950,13 +969,18 @@
     var source = NativeModule.getSource(this.id);
     source = NativeModule.wrap(source);
 
-    var fn = runInThisContext(source, {
-      filename: this.filename,
-      lineOffset: 0
-    });
-    fn(this.exports, NativeModule.require, this, this.filename);
+    this.loading = true;
+    try {
+      var fn = runInThisContext(source, {
+        filename: this.filename,
+        lineOffset: 0
+      });
+      fn(this.exports, NativeModule.require, this, this.filename);
 
-    this.loaded = true;
+      this.loaded = true;
+    } finally {
+      this.loading = false;
+    }
   };
 
   NativeModule.prototype.cache = function() {
