@@ -69,8 +69,6 @@ v8::Local<v8::Value> MakeCallback(Environment* env,
                                    int argc = 0,
                                    v8::Local<v8::Value>* argv = nullptr);
 
-bool KickNextTick();
-
 // Convert a struct sockaddr to a { address: '1.2.3.4', port: 1234 } JS object.
 // Sets address and port properties on the info object and returns it.
 // If |info| is omitted, a new object is returned.
@@ -82,6 +80,8 @@ v8::Local<v8::Object> AddressToJS(
 template <typename T, int (*F)(const typename T::HandleType*, sockaddr*, int*)>
 void GetSockOrPeerName(const v8::FunctionCallbackInfo<v8::Value>& args) {
   T* const wrap = Unwrap<T>(args.Holder());
+  if (wrap == nullptr)
+    return args.GetReturnValue().Set(UV_EBADF);
   CHECK(args[0]->IsObject());
   sockaddr_storage storage;
   int addrlen = sizeof(storage);
@@ -109,14 +109,11 @@ inline static int snprintf(char *buffer, size_t n, const char *format, ...) {
 #endif
 #endif
 
-#if defined(__x86_64__)
-# define BITS_PER_LONG 64
+#if defined(_MSC_VER) && _MSC_VER < 1900
+#define arraysize(a) (sizeof(a) / sizeof(*a))  // Workaround for VS 2013.
 #else
-# define BITS_PER_LONG 32
-#endif
-
-#ifndef ARRAY_SIZE
-# define ARRAY_SIZE(a) (sizeof((a)) / sizeof((a)[0]))
+template <typename T, size_t N>
+constexpr size_t arraysize(const T(&)[N]) { return N; }
 #endif
 
 #ifndef ROUND_UP
@@ -131,9 +128,11 @@ inline static int snprintf(char *buffer, size_t n, const char *format, ...) {
 # define NO_RETURN
 #endif
 
+enum ErrorHandlingMode { FATAL_ERROR, CONTEXTIFY_ERROR };
 void AppendExceptionLine(Environment* env,
                          v8::Local<v8::Value> er,
-                         v8::Local<v8::Message> message);
+                         v8::Local<v8::Message> message,
+                         enum ErrorHandlingMode mode = CONTEXTIFY_ERROR);
 
 NO_RETURN void FatalError(const char* location, const char* message);
 
@@ -163,24 +162,6 @@ inline bool IsBigEndian() {
   return GetEndianness() == kBigEndian;
 }
 
-// parse index for external array data
-inline MUST_USE_RESULT bool ParseArrayIndex(v8::Local<v8::Value> arg,
-                                            size_t def,
-                                            size_t* ret) {
-  if (arg->IsUndefined()) {
-    *ret = def;
-    return true;
-  }
-
-  int32_t tmp_i = arg->Int32Value();
-
-  if (tmp_i < 0)
-    return false;
-
-  *ret = static_cast<size_t>(tmp_i);
-  return true;
-}
-
 void ThrowError(v8::Isolate* isolate, const char* errmsg);
 void ThrowTypeError(v8::Isolate* isolate, const char* errmsg);
 void ThrowRangeError(v8::Isolate* isolate, const char* errmsg);
@@ -235,88 +216,94 @@ class ArrayBufferAllocator : public v8::ArrayBuffer::Allocator {
   inline void set_env(Environment* env) { env_ = env; }
 
   virtual void* Allocate(size_t size);  // Defined in src/node.cc
-  virtual void* AllocateUninitialized(size_t size) { return malloc(size); }
+  virtual void* AllocateUninitialized(size_t size)
+    { return node::Malloc(size); }
   virtual void Free(void* data, size_t) { free(data); }
 
  private:
   Environment* env_;
 };
 
+// Clear any domain and/or uncaughtException handlers to force the error's
+// propagation and shutdown the process. Use this to force the process to exit
+// by clearing all callbacks that could handle the error.
+void ClearFatalExceptionHandlers(Environment* env);
+
 enum NodeInstanceType { MAIN, WORKER };
 
 class NodeInstanceData {
-  public:
-    NodeInstanceData(NodeInstanceType node_instance_type,
-                     uv_loop_t* event_loop,
-                     int argc,
-                     const char** argv,
-                     int exec_argc,
-                     const char** exec_argv,
-                     bool use_debug_agent_flag)
-        : node_instance_type_(node_instance_type),
-          exit_code_(1),
-          event_loop_(event_loop),
-          argc_(argc),
-          argv_(argv),
-          exec_argc_(exec_argc),
-          exec_argv_(exec_argv),
-          use_debug_agent_flag_(use_debug_agent_flag) {
-      CHECK_NE(event_loop_, nullptr);
-    }
+ public:
+  NodeInstanceData(NodeInstanceType node_instance_type,
+                   uv_loop_t* event_loop,
+                   int argc,
+                   const char** argv,
+                   int exec_argc,
+                   const char** exec_argv,
+                   bool use_debug_agent_flag)
+      : node_instance_type_(node_instance_type),
+        exit_code_(1),
+        event_loop_(event_loop),
+        argc_(argc),
+        argv_(argv),
+        exec_argc_(exec_argc),
+        exec_argv_(exec_argv),
+        use_debug_agent_flag_(use_debug_agent_flag) {
+    CHECK_NE(event_loop_, nullptr);
+  }
 
-    uv_loop_t* event_loop() const {
-      return event_loop_;
-    }
+  uv_loop_t* event_loop() const {
+    return event_loop_;
+  }
 
-    int exit_code() {
-      CHECK(is_main());
-      return exit_code_;
-    }
+  int exit_code() {
+    CHECK(is_main());
+    return exit_code_;
+  }
 
-    void set_exit_code(int exit_code) {
-      CHECK(is_main());
-      exit_code_ = exit_code;
-    }
+  void set_exit_code(int exit_code) {
+    CHECK(is_main());
+    exit_code_ = exit_code;
+  }
 
-    bool is_main() {
-      return node_instance_type_ == MAIN;
-    }
+  bool is_main() {
+    return node_instance_type_ == MAIN;
+  }
 
-    bool is_worker() {
-      return node_instance_type_ == WORKER;
-    }
+  bool is_worker() {
+    return node_instance_type_ == WORKER;
+  }
 
-    int argc() {
-      return argc_;
-    }
+  int argc() {
+    return argc_;
+  }
 
-    const char** argv() {
-      return argv_;
-    }
+  const char** argv() {
+    return argv_;
+  }
 
-    int exec_argc() {
-      return exec_argc_;
-    }
+  int exec_argc() {
+    return exec_argc_;
+  }
 
-    const char** exec_argv() {
-      return exec_argv_;
-    }
+  const char** exec_argv() {
+    return exec_argv_;
+  }
 
-    bool use_debug_agent() {
-      return is_main() && use_debug_agent_flag_;
-    }
+  bool use_debug_agent() {
+    return is_main() && use_debug_agent_flag_;
+  }
 
-  private:
-    const NodeInstanceType node_instance_type_;
-    int exit_code_;
-    uv_loop_t* const event_loop_;
-    const int argc_;
-    const char** argv_;
-    const int exec_argc_;
-    const char** exec_argv_;
-    const bool use_debug_agent_flag_;
+ private:
+  const NodeInstanceType node_instance_type_;
+  int exit_code_;
+  uv_loop_t* const event_loop_;
+  const int argc_;
+  const char** argv_;
+  const int exec_argc_;
+  const char** exec_argv_;
+  const bool use_debug_agent_flag_;
 
-    DISALLOW_COPY_AND_ASSIGN(NodeInstanceData);
+  DISALLOW_COPY_AND_ASSIGN(NodeInstanceData);
 };
 
 namespace Buffer {
