@@ -1,8 +1,8 @@
 'use strict';
 // Flags: --expose-gc
 
-require('../../common');
-var binding = require('./build/Release/binding');
+const common = require('../../common');
+const binding = require(`./build/${common.buildType}/binding`);
 
 function check(size) {
   var buf = binding.alloc(size);
