@@ -4,7 +4,7 @@
 
 var common = require('../common');
 if (!common.hasCrypto) {
-  console.log('1..0 # Skipped: missing crypto');
+  common.skip('missing crypto');
   return;
 }
 
