@@ -1,16 +1,16 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-var cluster = require('cluster');
-var net = require('net');
+const common = require('../common');
+const assert = require('assert');
+const cluster = require('cluster');
+const net = require('net');
 
 if (common.isWindows) {
-  console.log('1..0 # Skipped: not reliable on Windows');
+  common.skip('not reliable on Windows');
   return;
 }
 
 if (process.getuid() === 0) {
-  console.log('1..0 # Skipped: as this test should not be run as `root`');
+  common.skip('as this test should not be run as `root`');
   return;
 }
 
@@ -18,14 +18,13 @@ if (cluster.isMaster) {
   // Master opens and binds the socket and shares it with the worker.
   cluster.schedulingPolicy = cluster.SCHED_NONE;
   cluster.fork().on('exit', common.mustCall(function(exitCode) {
-    assert.equal(exitCode, 0);
+    assert.strictEqual(exitCode, 0);
   }));
-}
-else {
-  var s = net.createServer(common.fail);
+} else {
+  const s = net.createServer(common.fail);
   s.listen(42, common.fail.bind(null, 'listen should have failed'));
   s.on('error', common.mustCall(function(err) {
-    assert.equal(err.code, 'EACCES');
+    assert.strictEqual(err.code, 'EACCES');
     process.disconnect();
   }));
 }
