@@ -1,10 +1,13 @@
 'use strict';
 require('../common');
-var assert = require('assert');
-var Stream = require('stream');
-var Console = require('console').Console;
+const assert = require('assert');
+const Stream = require('stream');
+const Console = require('console').Console;
 var called = false;
 
+const out = new Stream();
+const err = new Stream();
+
 // ensure the Console instance doesn't write to the
 // process' "stdout" or "stderr" streams
 process.stdout.write = process.stderr.write = function() {
@@ -12,7 +15,7 @@ process.stdout.write = process.stderr.write = function() {
 };
 
 // make sure that the "Console" function exists
-assert.equal('function', typeof Console);
+assert.strictEqual('function', typeof Console);
 
 // make sure that the Console constructor throws
 // when not given a writable stream instance
@@ -20,15 +23,19 @@ assert.throws(function() {
   new Console();
 }, /Console expects a writable stream/);
 
-var out = new Stream();
-var err = new Stream();
-out.writable = err.writable = true;
+// Console constructor should throw if stderr exists but is not writable
+assert.throws(function() {
+  out.write = function() {};
+  err.write = undefined;
+  new Console(out, err);
+}, /Console expects writable stream instances/);
+
 out.write = err.write = function(d) {};
 
 var c = new Console(out, err);
 
 out.write = err.write = function(d) {
-  assert.equal(d, 'test\n');
+  assert.strictEqual(d, 'test\n');
   called = true;
 };
 
@@ -41,7 +48,7 @@ c.error('test');
 assert(called);
 
 out.write = function(d) {
-  assert.equal('{ foo: 1 }\n', d);
+  assert.strictEqual('{ foo: 1 }\n', d);
   called = true;
 };
 
@@ -53,7 +60,12 @@ assert(called);
 called = 0;
 out.write = function(d) {
   called++;
-  assert.equal(d, called + ' ' + (called - 1) + ' [ 1, 2, 3 ]\n');
+  assert.strictEqual(d, called + ' ' + (called - 1) + ' [ 1, 2, 3 ]\n');
 };
 [1, 2, 3].forEach(c.log);
-assert.equal(3, called);
+assert.strictEqual(3, called);
+
+// Console() detects if it is called without `new` keyword
+assert.doesNotThrow(function() {
+  Console(out, err);
+});
