@@ -3,7 +3,7 @@ var common = require('../common');
 var assert = require('assert');
 
 if (!common.hasCrypto) {
-  console.log('1..0 # Skipped: missing crypto');
+  common.skip('missing crypto');
   return;
 }
 var crypto = require('crypto');
@@ -19,18 +19,20 @@ var spkacPem = fs.readFileSync(common.fixturesDir + '/spkac.pem');
 
 var certificate = new crypto.Certificate();
 
-assert.equal(certificate.verifySpkac(spkacValid), true);
-assert.equal(certificate.verifySpkac(spkacFail), false);
+assert.strictEqual(certificate.verifySpkac(spkacValid), true);
+assert.strictEqual(certificate.verifySpkac(spkacFail), false);
 
-assert.equal(stripLineEndings(certificate.exportPublicKey(spkacValid)
-							  .toString('utf8')),
-			 stripLineEndings(spkacPem.toString('utf8')));
-assert.equal(certificate.exportPublicKey(spkacFail), '');
+assert.strictEqual(
+  stripLineEndings(certificate.exportPublicKey(spkacValid).toString('utf8')),
+  stripLineEndings(spkacPem.toString('utf8'))
+);
+assert.strictEqual(certificate.exportPublicKey(spkacFail), '');
 
-assert.equal(certificate.exportChallenge(spkacValid)
-			 .toString('utf8'),
-			 'fb9ab814-6677-42a4-a60c-f905d1a6924d');
-assert.equal(certificate.exportChallenge(spkacFail), '');
+assert.strictEqual(
+  certificate.exportChallenge(spkacValid).toString('utf8'),
+  'fb9ab814-6677-42a4-a60c-f905d1a6924d'
+);
+assert.strictEqual(certificate.exportChallenge(spkacFail), '');
 
 function stripLineEndings(obj) {
   return obj.replace(/\n/g, '');
