@@ -1,5 +1,5 @@
 'use strict';
-var common = require('../common');
+require('../common');
 var assert = require('assert');
 var http = require('http');
 
@@ -17,7 +17,7 @@ var server = http.createServer(function(req, res) {
 function get(path, callback) {
   return http.get({
     host: 'localhost',
-    port: common.PORT,
+    port: server.address().port,
     agent: agent,
     path: path
   }, callback);
@@ -30,12 +30,12 @@ function done() {
   }
   var freepool = agent.freeSockets[Object.keys(agent.freeSockets)[0]];
   assert.equal(freepool.length, 2,
-      'expect keep 2 free sockets, but got ' + freepool.length);
+               'expect keep 2 free sockets, but got ' + freepool.length);
   agent.destroy();
   server.close();
 }
 
-server.listen(common.PORT, function() {
+server.listen(0, function() {
   get('/1', function(res) {
     assert.equal(res.statusCode, 200);
     res.resume();
