@@ -1,5 +1,5 @@
 'use strict';
-var common = require('../common');
+require('../common');
 var assert = require('assert');
 var http = require('http');
 
@@ -16,7 +16,7 @@ var httpServer = http.createServer(function(req, res) {
 
   res.on('finish', function() {
     sawFinish = true;
-    assert(typeof(req.connection.bytesWritten) === 'number');
+    assert(typeof req.connection.bytesWritten === 'number');
     assert(req.connection.bytesWritten > 0);
   });
   res.writeHead(200, { 'Content-Type': 'text/plain' });
@@ -36,7 +36,7 @@ var httpServer = http.createServer(function(req, res) {
   res.end(body);
 });
 
-httpServer.listen(common.PORT, function() {
-  http.get({ port: common.PORT });
+httpServer.listen(0, function() {
+  http.get({ port: this.address().port });
 });
 
