@@ -1,5 +1,5 @@
 'use strict';
-var common = require('../common');
+require('../common');
 var assert = require('assert');
 var http = require('http');
 
@@ -23,11 +23,11 @@ var server = http.createServer(function(req, res) {
     res.end();
   });
 });
-server.listen(common.PORT);
+server.listen(0);
 
 server.on('listening', function() {
   var req = http.request({
-    port: common.PORT,
+    port: this.address().port,
     method: 'POST',
     path: '/'
   }, function(res) {
