@@ -1,10 +1,10 @@
 'use strict';
-var common = require('../common');
-var http = require('http'),
-    assert = require('assert');
+const common = require('../common');
+const http = require('http');
+const assert = require('assert');
 
 if (!common.hasMultiLocalhost()) {
-  console.log('1..0 # Skipped: platform-specific test.');
+  common.skip('platform-specific test.');
   return;
 }
 
@@ -19,9 +19,9 @@ var server = http.createServer(function(req, res) {
   req.resume();
 });
 
-server.listen(common.PORT, '127.0.0.1', function() {
+server.listen(0, '127.0.0.1', function() {
   var options = { host: 'localhost',
-    port: common.PORT,
+    port: this.address().port,
     path: '/',
     method: 'GET',
     localAddress: '127.0.0.2' };
