@@ -1,5 +1,5 @@
 'use strict';
-var common = require('../common');
+require('../common');
 var assert = require('assert');
 var net = require('net');
 var http = require('http');
@@ -9,8 +9,8 @@ var body = '';
 var server = net.createServer(function(socket) {
   // Neither Content-Length nor Connection
   socket.end('HTTP/1.1 200 ok\r\n\r\nHello');
-}).listen(common.PORT, function() {
-  http.get({port: common.PORT}, function(res) {
+}).listen(0, function() {
+  http.get({port: this.address().port}, function(res) {
     res.setEncoding('utf8');
     res.on('data', function(chunk) {
       body += chunk;
