@@ -1,5 +1,5 @@
 'use strict';
-var common = require('../common');
+require('../common');
 var assert = require('assert');
 var net = require('net');
 var http = require('http');
@@ -7,8 +7,6 @@ var http = require('http');
 // Test that the DELETE, PATCH and PURGE verbs get passed through correctly
 
 ['DELETE', 'PATCH', 'PURGE'].forEach(function(method, index) {
-  var port = common.PORT + index;
-
   var server_response = '';
   var received_method = null;
 
@@ -19,10 +17,10 @@ var http = require('http');
     res.write('world\n');
     res.end();
   });
-  server.listen(port);
+  server.listen(0);
 
   server.on('listening', function() {
-    var c = net.createConnection(port);
+    var c = net.createConnection(this.address().port);
 
     c.setEncoding('utf8');
 
