@@ -1,5 +1,5 @@
 'use strict';
-var common = require('../common');
+require('../common');
 var assert = require('assert');
 var http = require('http');
 var net = require('net');
@@ -10,12 +10,12 @@ var s = http.createServer(function(req, res) {
   res.end('');
 });
 
-s.listen(common.PORT, test);
+s.listen(0, test);
 
 
 function test() {
   var bufs = [];
-  var client = net.connect(common.PORT, function() {
+  var client = net.connect(this.address().port, function() {
     client.write('GET / HTTP/1.1\r\nConnection: close\r\n\r\n');
   });
   client.on('data', function(chunk) {
