@@ -7,16 +7,16 @@ var conns = 0;
 
 var server = net.createServer(function(socket) {
   conns++;
-  assert.equal(common.localhostIPv4, socket.localAddress);
-  assert.equal(socket.localPort, common.PORT);
+  assert.equal(socket.localAddress, common.localhostIPv4);
+  assert.equal(socket.localPort, this.address().port);
   socket.on('end', function() {
     server.close();
   });
   socket.resume();
 });
 
-server.listen(common.PORT, common.localhostIPv4, function() {
-  var client = net.createConnection(common.PORT, common.localhostIPv4);
+server.listen(0, common.localhostIPv4, function() {
+  var client = net.createConnection(this.address().port, common.localhostIPv4);
   client.on('connect', function() {
     client.end();
   });
