@@ -1,5 +1,5 @@
 'use strict';
-var common = require('../common');
+require('../common');
 var assert = require('assert');
 var net = require('net');
 var TCPWrap = process.binding('tcp_wrap').TCP;
@@ -25,13 +25,13 @@ TCPWrap.prototype.unref = function() {
   assert.equal(refCount, -1);
 };
 
-echoServer.listen(common.PORT);
+echoServer.listen(0);
 
 echoServer.on('listening', function() {
   var sock = new net.Socket();
   sock.unref();
   sock.ref();
-  sock.connect(common.PORT);
+  sock.connect(this.address().port);
   sock.on('end', function() {
     assert.equal(refCount, 0);
     echoServer.close();
