@@ -7,7 +7,7 @@ const net = require('net');
 process.chdir(common.fixturesDir);
 const repl = require('repl');
 
-const server = net.createServer(conn => {
+const server = net.createServer((conn) => {
   repl.start('', conn).on('exit', () => {
     conn.destroy();
     server.close();
@@ -15,14 +15,15 @@ const server = net.createServer(conn => {
 });
 
 const host = common.localhostIPv4;
-const port = common.PORT;
+const port = 0;
 const options = { host, port };
 
 var answer = '';
 server.listen(options, function() {
+  options.port = this.address().port;
   const conn = net.connect(options);
   conn.setEncoding('utf8');
-  conn.on('data', data => answer += data);
+  conn.on('data', (data) => answer += data);
   conn.write('require("baz")\n.exit\n');
 });
 
