@@ -1,36 +1,36 @@
 /* eslint-disable max-len, strict */
-var common = require('../common');
-var assert = require('assert');
+const common = require('../common');
+const assert = require('assert');
 
 common.globalCheck = false;
 common.refreshTmpDir();
 
-var net = require('net'),
-    repl = require('repl'),
-    message = 'Read, Eval, Print Loop',
-    prompt_unix = 'node via Unix socket> ',
-    prompt_tcp = 'node via TCP socket> ',
-    prompt_multiline = '... ',
-    prompt_npm = 'npm should be run outside of the ' +
-                 'node repl, in your normal shell.\n' +
-                 '(Press Control-D to exit.)\n',
-    expect_npm = prompt_npm + prompt_unix,
-    server_tcp, server_unix, client_tcp, client_unix, timer;
+const net = require('net');
+const repl = require('repl');
+const message = 'Read, Eval, Print Loop';
+const prompt_unix = 'node via Unix socket> ';
+const prompt_tcp = 'node via TCP socket> ';
+const prompt_multiline = '... ';
+const prompt_npm = 'npm should be run outside of the ' +
+                   'node repl, in your normal shell.\n' +
+                   '(Press Control-D to exit.)\n';
+const expect_npm = prompt_npm + prompt_unix;
+var server_tcp, server_unix, client_tcp, client_unix, replServer;
 
 
 // absolute path to test/fixtures/a.js
-var moduleFilename = require('path').join(common.fixturesDir, 'a');
+const moduleFilename = require('path').join(common.fixturesDir, 'a');
 
 console.error('repl test');
 
 // function for REPL to run
-invoke_me = function(arg) {
+global.invoke_me = function(arg) {
   return 'invoked ' + arg;
 };
 
 function send_expect(list) {
   if (list.length > 0) {
-    var cur = list.shift();
+    const cur = list.shift();
 
     console.error('sending ' + JSON.stringify(cur.send));
 
@@ -45,12 +45,19 @@ function send_expect(list) {
 function clean_up() {
   client_tcp.end();
   client_unix.end();
-  clearTimeout(timer);
+}
+
+function strict_mode_error_test() {
+  send_expect([
+    { client: client_unix, send: 'ref = 1',
+      expect: /^ReferenceError:\sref\sis\snot\sdefined\n\s+at\srepl:1:5/ },
+  ]);
 }
 
 function error_test() {
   // The other stuff is done so reuse unix socket
   var read_buffer = '';
+  var run_strict_test = true;
   client_unix.removeAllListeners('data');
 
   client_unix.on('data', function(data) {
@@ -65,13 +72,17 @@ function error_test() {
       if (read_buffer !== client_unix.expect) {
         var expect = client_unix.expect;
         if (expect === prompt_multiline)
-          expect = /[\.]{3} /;
+          expect = /[.]{3} /;
         assert.ok(read_buffer.match(expect));
         console.error('match');
       }
       read_buffer = '';
       if (client_unix.list && client_unix.list.length > 0) {
         send_expect(client_unix.list);
+      } else if (run_strict_test) {
+        replServer.replMode = repl.REPL_MODE_STRICT;
+        run_strict_test = false;
+        strict_mode_error_test();
       } else {
         console.error('End of Error test, running TCP test.');
         tcp_test();
@@ -83,6 +94,10 @@ function error_test() {
       read_buffer = '';
       if (client_unix.list && client_unix.list.length > 0) {
         send_expect(client_unix.list);
+      } else if (run_strict_test) {
+        replServer.replMode = repl.REPL_MODE_STRICT;
+        run_strict_test = false;
+        strict_mode_error_test();
       } else {
         console.error('End of Error test, running TCP test.\n');
         tcp_test();
@@ -282,6 +297,19 @@ function error_test() {
     // access to internal modules without the --expose_internals flag.
     { client: client_unix, send: 'require("internal/repl")',
       expect: /^Error: Cannot find module 'internal\/repl'/ },
+    // REPL should handle quotes within regexp literal in multiline mode
+    { client: client_unix, send: "function x(s) {\nreturn s.replace(/'/,'');\n}",
+      expect: prompt_multiline + prompt_multiline +
+            'undefined\n' + prompt_unix },
+    { client: client_unix, send: "function x(s) {\nreturn s.replace(/\'/,'');\n}",
+      expect: prompt_multiline + prompt_multiline +
+            'undefined\n' + prompt_unix },
+    { client: client_unix, send: 'function x(s) {\nreturn s.replace(/"/,"");\n}',
+      expect: prompt_multiline + prompt_multiline +
+            'undefined\n' + prompt_unix },
+    { client: client_unix, send: 'function x(s) {\nreturn s.replace(/.*/,"");\n}',
+      expect: prompt_multiline + prompt_multiline +
+            'undefined\n' + prompt_unix },
   ]);
 }
 
@@ -296,14 +324,14 @@ function tcp_test() {
     repl.start(prompt_tcp, socket);
   });
 
-  server_tcp.listen(common.PORT, function() {
+  server_tcp.listen(0, function() {
     var read_buffer = '';
 
-    client_tcp = net.createConnection(common.PORT);
+    client_tcp = net.createConnection(this.address().port);
 
     client_tcp.on('connect', function() {
-      assert.equal(true, client_tcp.readable);
-      assert.equal(true, client_tcp.writable);
+      assert.strictEqual(true, client_tcp.readable);
+      assert.strictEqual(true, client_tcp.writable);
 
       send_expect([
         { client: client_tcp, send: '',
@@ -356,12 +384,13 @@ function unix_test() {
       socket.end();
     });
 
-    repl.start({
+    replServer = repl.start({
       prompt: prompt_unix,
       input: socket,
       output: socket,
       useGlobal: true
-    }).context.message = message;
+    });
+    replServer.context.message = message;
   });
 
   server_unix.on('listening', function() {
@@ -370,8 +399,8 @@ function unix_test() {
     client_unix = net.createConnection(common.PIPE);
 
     client_unix.on('connect', function() {
-      assert.equal(true, client_unix.readable);
-      assert.equal(true, client_unix.writable);
+      assert.strictEqual(true, client_unix.readable);
+      assert.strictEqual(true, client_unix.writable);
 
       send_expect([
         { client: client_unix, send: '',
@@ -419,7 +448,3 @@ function unix_test() {
 }
 
 unix_test();
-
-timer = setTimeout(function() {
-  assert.fail(null, null, 'Timeout');
-}, 5000);
