@@ -4,8 +4,8 @@
  */
 const common = require('../common');
 
-const TEST_DURATION = common.platformTimeout(100);
-const N = 5;
+const TEST_DURATION = common.platformTimeout(1000);
+const N = 3;
 var nbIntervalFired = 0;
 
 const keepOpen = setTimeout(() => {
@@ -20,7 +20,7 @@ const timer = setInterval(() => {
     timer._onTimeout = () => {
       throw new Error('Unrefd interval fired after being cleared.');
     };
-    setImmediate(() => clearTimeout(keepOpen));
+    clearTimeout(keepOpen);
   }
 }, 1);
 
