@@ -7,7 +7,7 @@ var fs = require('fs');
 var constants = require('constants');
 
 if (!common.hasCrypto) {
-  console.log('1..0 # Skipped: missing crypto');
+  common.skip('missing crypto');
   return;
 }
 
@@ -35,9 +35,9 @@ server.on('resumeSession', function(id, cb) {
   next();
 });
 
-server.listen(common.PORT, function() {
+server.listen(0, function() {
   var clientOpts = {
-    port: common.PORT,
+    port: this.address().port,
     rejectUnauthorized: false,
     session: false
   };
