@@ -1,26 +1,26 @@
 'use strict';
-var common = require('../common');
+const common = require('../common');
 
 if (!common.hasCrypto) {
-  console.log('1..0 # Skipped: missing crypto');
+  common.skip('missing crypto');
   return;
 }
-var tls = require('tls');
-var stream = require('stream');
+const tls = require('tls');
+const stream = require('stream');
 
-var delay = new stream.Duplex({
+const delay = new stream.Duplex({
   read: function read() {
   },
   write: function write(data, enc, cb) {
     console.log('pending');
-    setTimeout(function() {
+    setImmediate(function() {
       console.log('done');
       cb();
-    }, 200);
+    });
   }
 });
 
-var secure = tls.connect({
+const secure = tls.connect({
   socket: delay
 });
 setImmediate(function() {
