@@ -3,7 +3,7 @@ var common = require('../common');
 var assert = require('assert');
 
 if (!common.hasCrypto) {
-  console.log('1..0 # Skipped: missing crypto');
+  common.skip('missing crypto');
   return;
 }
 var tls = require('tls');
@@ -20,9 +20,9 @@ var server = net.Server(function(raw) {
     success = true;
   });
   require('_tls_legacy').pipe(pair, raw);
-}).listen(common.PORT, function() {
+}).listen(0, function() {
   tls.connect({
-    port: common.PORT,
+    port: this.address().port,
     rejectUnauthorized: false,
     servername: 'server'
   }, function() {
