@@ -11,27 +11,26 @@ var nevents = 0;
 
 try {
   fs.unlinkSync(FILENAME);
-}
-catch (e) {
+} catch (e) {
   // swallow
 }
 
-fs.watchFile(FILENAME, {interval:TIMEOUT - 250}, function(curr, prev) {
+fs.watchFile(FILENAME, {interval: TIMEOUT - 250}, function(curr, prev) {
   console.log([curr, prev]);
   switch (++nevents) {
-  case 1:
-    assert.equal(common.fileExists(FILENAME), false);
-    break;
-  case 2:
-  case 3:
-    assert.equal(common.fileExists(FILENAME), true);
-    break;
-  case 4:
-    assert.equal(common.fileExists(FILENAME), false);
-    fs.unwatchFile(FILENAME);
-    break;
-  default:
-    assert(0);
+    case 1:
+      assert.equal(common.fileExists(FILENAME), false);
+      break;
+    case 2:
+    case 3:
+      assert.equal(common.fileExists(FILENAME), true);
+      break;
+    case 4:
+      assert.equal(common.fileExists(FILENAME), false);
+      fs.unwatchFile(FILENAME);
+      break;
+    default:
+      assert(0);
   }
 });
 
