@@ -1,24 +1,4 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
-
+'use strict';
 var common = require('../common');
 var assert = require('assert');
 var path = require('path');
@@ -27,8 +7,8 @@ var fs = require('fs');
 var watchSeenOne = 0;
 var watchSeenTwo = 0;
 var watchSeenThree = 0;
+var watchSeenFour = 0;
 
-var startDir = process.cwd();
 var testDir = common.tmpDir;
 
 var filenameOne = 'watch.txt';
@@ -40,14 +20,17 @@ var filepathTwoAbs = path.join(testDir, filenameTwo);
 
 var filenameThree = 'charm'; // because the third time is
 
+var filenameFour = 'get';
 
 process.on('exit', function() {
   fs.unlinkSync(filepathOne);
   fs.unlinkSync(filepathTwoAbs);
   fs.unlinkSync(filenameThree);
+  fs.unlinkSync(filenameFour);
   assert.equal(1, watchSeenOne);
   assert.equal(2, watchSeenTwo);
   assert.equal(1, watchSeenThree);
+  assert.equal(1, watchSeenFour);
 });
 
 
@@ -126,3 +109,22 @@ assert.doesNotThrow(
 setTimeout(function() {
   fs.writeFileSync(filenameThree, 'pardner');
 }, 1000);
+
+setTimeout(function() {
+  fs.writeFileSync(filenameFour, 'hey');
+}, 200);
+
+setTimeout(function() {
+  fs.writeFileSync(filenameFour, 'hey');
+}, 500);
+
+assert.doesNotThrow(
+    function() {
+      function a(curr, prev) {
+        ++watchSeenFour;
+        assert.equal(1, watchSeenFour);
+        fs.unwatchFile('.' + path.sep + filenameFour, a);
+      }
+      fs.watchFile(filenameFour, a);
+    }
+);
