@@ -25,7 +25,7 @@ tls.createServer({
 
   var options = { rejectUnauthorized: false };
   tls.connect(common.PORT, '127.0.0.1', options, function() {
-    assert(junk.length != 0);  // keep reference alive
+    assert.notStrictEqual(junk.length, 0);  // keep reference alive
     setTimeout(done, 10);
     gc();
   });
