@@ -6,22 +6,16 @@ var common = require('../common');
 var assert = require('assert');
 
 if (!common.hasCrypto) {
-  console.log('1..0 # Skipped: missing crypto');
+  common.skip('missing crypto');
   return;
 }
 var tls = require('tls');
 var fs = require('fs');
 
-
-var body = '';
-
 process.stdout.write('build body...');
-for (var i = 0; i < 1024 * 1024; i++) {
-  body += 'hello world\n';
-}
+var body = 'hello world\n'.repeat(1024 * 1024);
 process.stdout.write('done\n');
 
-
 var options = {
   key: fs.readFileSync(common.fixturesDir + '/keys/agent2-key.pem'),
   cert: fs.readFileSync(common.fixturesDir + '/keys/agent2-cert.pem')
